@@ -7,12 +7,20 @@
  ***************************************************************************/
 
 /***************************************************************************
- *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
+ *   This program is distributed in the hope that it will be useful, but   *
+ *   WITHOUT ANY WARRANTY; without even the implied warranty of            *
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
+ *   General Public License for more details.                              *
+ *                                                                         *
+ *   You should have received a copy of the GNU General Public License     *
+ *   along with this program; if not, write to the                         *
+ *   Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
+ *   MA  02111-1307 USA                                                    *
  ***************************************************************************/
 
 
@@ -31,10 +39,14 @@
 #include <kapplication.h>
 #include <kstandarddirs.h>
 #include <kmessagebox.h>
+#include <kiconloader.h>
+#include <klineedit.h>
+#include <kcombobox.h>
 
 #ifdef __FreeBSD__
-#include <kprocess.h>
 #include <sys/stat.h>
+#include <fcntl.h>
+#include <errno.h>
 #endif
 
 // application specific includes
@@ -42,17 +54,12 @@
 #include "smb4kdefs.h"
 #include "smb4kglobal.h"
 
-using namespace Smb4K_Global;
-
-
-Smb4KPasswordHandler *Smb4KPasswordHandler::m_this_class = 0L;
+using namespace Smb4KGlobal;
 
 
 Smb4KPasswordHandler::Smb4KPasswordHandler( QObject *parent, const char *name )
 : QObject( parent, name )
 {
-  m_this_class = this;
-
   config()->setGroup( "Authentication" );
 
   m_auth = NULL;
@@ -66,13 +73,18 @@ Smb4KPasswordHandler::Smb4KPasswordHandler( QObject *parent, const char *name )
   }
 
   config()->sync();
-
-//   open_close_wallet();
 }
 
 
 Smb4KPasswordHandler::~Smb4KPasswordHandler()
 {
+  for ( QValueList<Smb4KAuthInfo *>::Iterator it = m_auth_list.begin(); it != m_auth_list.end(); ++it )
+  {
+    delete *it;
+  }
+
+  m_auth_list.clear();
+
   m_wallet->disconnectApplication( KWallet::Wallet::NetworkWallet(), kapp->name() );
   m_wallet->closeWallet( KWallet::Wallet::NetworkWallet(), false );
   delete m_wallet;
@@ -107,6 +119,10 @@ void Smb4KPasswordHandler::open_close_wallet()
         delete m_wallet;
         m_wallet = NULL;
 
+        // Do not remove the following line or the entry won't be written
+        // properly. This seems to have to do with the transition to the global
+        // config object.
+        config()->setGroup( "Authentication" );
         config()->writeEntry( "Use Wallet", false );
         config()->sync();
 
@@ -132,34 +148,128 @@ bool Smb4KPasswordHandler::askpass( const QString &workgroup, const QString &hos
 {
   m_auth = readAuth( workgroup, host, share );
 
-  m_dlg = new AskPass::AskPass( m_auth, desc, 0, "AskPassDlg" );
+  // Set up the askpass dialog:
+
+  m_dlg = new KDialogBase( KDialogBase::Plain, i18n( "Authentication" ), KDialogBase::Ok|KDialogBase::Cancel,
+                           KDialogBase::Ok, 0, "AskPassDlg", false, true );
+
+  QFrame *frame = m_dlg->plainPage();
+  QGridLayout *layout = new QGridLayout( frame );
+  layout->setSpacing( 10 );
+  layout->setMargin( 0 );
+
+  QLabel *pixmap_label = new QLabel( frame );
+  pixmap_label->setPixmap( DesktopIcon( "identity" ) );
+  pixmap_label->adjustSize();
+
+  layout->addWidget( pixmap_label, 0, 0, Qt::AlignCenter );
+
+  QString message;
+
+  switch ( desc )
+  {
+    case NewData:
+      break;
+    case AccessDenied:
+      message = i18n( "The access was denied. " );
+      break;
+    case BadPassword:
+      message = i18n( "The password is not correct. " );
+      break;
+    case PermDenied:
+      message = i18n( "The permission was denied. " );
+      break;
+    case AuthError:
+      message = i18n( "An authentication error occurred. " );
+      break;
+    case LogonFailure:
+      message = i18n( "The logon failed. " );
+      break;
+    default:
+      break;
+  }
+
+  message.append( i18n( "Please enter authentication data for %1." ).arg(
+                  m_auth->share().stripWhiteSpace().isEmpty() ? m_auth->host() :
+                  "//"+m_auth->host()+"/"+m_auth->share() ) );
+
+  QLabel *message_label = new QLabel( frame );
+  message_label->setText( message.stripWhiteSpace() );
+  message_label->setTextFormat( Qt::RichText );
+
+  layout->addWidget( message_label, 0, 1, 0 );
+
+  QLabel *user_label = new QLabel( i18n( "User:" ), frame );
+  layout->addWidget( user_label, 1, 0, 0 );
+
+  KLineEdit *user_edit = NULL;
+  KComboBox *user_combo = NULL;
+
+  if ( QString::compare( m_auth->share(), "homes" ) != 0 )
+  {
+    user_edit = new KLineEdit( frame, "AskPassUserEdit" );
+    layout->addWidget( user_edit, 1, 1, 0 );
+  }
+  else
+  {
+    user_combo = new KComboBox( frame, "AskPassUserCombo" );
+    user_combo->setEditable( true );
+    layout->addWidget( user_combo, 1, 1, 0 );
+  }
+
+  QLabel *password_label = new QLabel( i18n( "Password:" ), frame );
+  layout->addWidget( password_label, 2, 0, 0 );
+
+  KLineEdit *pass_edit = new KLineEdit( frame, "AskPassPasswordEdit" );
+  pass_edit->setEchoMode( KLineEdit::Password );
+  layout->addWidget( pass_edit, 2, 1, 0 );
+
+  m_dlg->setMainWidget( frame );
+  frame->setFixedWidth( 350 );
+  m_dlg->setFixedSize( m_dlg->sizeHint() );
+  m_dlg->enableButtonOK( false );
+
+  // Since we have to allow empty passwords, we will only connect
+  // the edit line for the user to enable/disable the OK button.
+  if ( user_edit )
+  {
+    connect( user_edit, SIGNAL( textChanged( const QString & ) ),
+             this,      SLOT( slotEnableOKButton( const QString& ) ) );
+  }
+  else
+  {
+    connect( user_combo, SIGNAL( textChanged( const QString & ) ),
+             this,       SLOT( slotEnableOKButton( const QString& ) ) );
+  }
+
+  // Process the authentication data:
 
   if ( QString::compare( share, "homes" ) != 0 )
   {
-    m_dlg->userEdit()->setText( m_auth->user() );
-    m_dlg->passwdEdit()->setText( m_auth->password() );
+    user_edit->setText( m_auth->user() );
+    pass_edit->setText( m_auth->password() );
 
     if ( m_auth->user().isEmpty() )
     {
-      m_dlg->userEdit()->setFocus();
+      user_edit->setFocus();
     }
     else
     {
-      m_dlg->passwdEdit()->setFocus();
+      pass_edit->setFocus();
     }
   }
   else
   {
     config()->setGroup( "Homes Shares" );
-
     QStringList list = config()->readListEntry( host.upper() );
 
-    m_dlg->userCombo()->insertStringList( list );
-    m_dlg->userCombo()->setCurrentText( QString::null );
+    user_combo->insertStringList( list );
+    user_combo->setCurrentText( QString::null );
 
-    connect( m_dlg->userCombo(), SIGNAL( activated( const QString & ) ), this, SLOT( slotGetPassword( const QString & ) ) );
+    connect( user_combo, SIGNAL( activated( const QString & ) ),
+             this,       SLOT( slotGetPassword( const QString & ) ) );
 
-    m_dlg->userCombo()->setFocus();
+    user_combo->setFocus();
   }
 
   bool ok = false;
@@ -168,28 +278,28 @@ bool Smb4KPasswordHandler::askpass( const QString &workgroup, const QString &hos
   {
     if ( QString::compare( share, "homes" ) != 0 )
     {
-      QString user = m_dlg->userEdit()->text();
-      QString pass = m_dlg->passwdEdit()->text();
+      QString user = user_edit->text();
+      QString pass = pass_edit->text();
 
       m_auth->setUser( user );
       m_auth->setPassword( pass );
-
-      writeAuth( m_auth );
     }
     else
     {
-      QString user = m_dlg->userCombo()->currentText();
-      QString pass = m_dlg->passwdEdit()->text();
+      QString user = user_combo->currentText();
+      QString pass = pass_edit->text();
 
       m_auth->setUser( user );
       m_auth->setPassword( pass );
-
-      writeAuth( m_auth );
     }
 
+    writeAuth( m_auth );
+
     ok = true;
   }
 
+  // Clean up:
+
   delete m_dlg;
   m_dlg = NULL;
 
@@ -369,12 +479,30 @@ void Smb4KPasswordHandler::writeAuth( Smb4KAuthInfo *auth )
 
   if ( m_wallet && m_wallet->isOpen() )
   {
+    // Remove old entry:
     QString key;
     key.append( auth->workgroup().isEmpty() ? "*" : auth->workgroup().upper() );
     key.append( ":" );
     key.append( auth->host().upper() );
     key.append( ":" );
     key.append( auth->share().isEmpty() ? "*" : auth->share().upper() );
+
+    QStringList entries = m_wallet->entryList();
+
+    for ( QStringList::ConstIterator it = entries.begin(); it != entries.end(); ++it )
+    {
+      if ( (*it).stripWhiteSpace().startsWith( key ) )
+      {
+        m_wallet->removeEntry( *it );
+
+        break;
+      }
+      else
+      {
+        continue;
+      }
+    }
+
     key.append( ":" );
     key.append( auth->user() );
 
@@ -579,8 +707,10 @@ void Smb4KPasswordHandler::writeToSMBConfFile( Smb4KAuthInfo *auth )
   KProcess *p = new KProcess();
   p->setUseShell( true );
 
-  connect( p, SIGNAL( receivedStdout( KProcess *, char *, int ) ), this, SLOT( slotReceivePassword( KProcess *, char *, int ) ) );
-  connect( p, SIGNAL( processExited( KProcess * ) ),               this, SLOT( slotWritePassword( KProcess * ) ) );
+  connect( p,    SIGNAL( receivedStdout( KProcess *, char *, int ) ),
+           this, SLOT( slotReceivePassword( KProcess *, char *, int ) ) );
+  connect( p,    SIGNAL( processExited( KProcess * ) ),
+           this, SLOT( slotWritePassword( KProcess * ) ) );
 
   *p << QString( "smbutil crypt %1" ).arg( m_nsmbrc_auth.password() );
 
@@ -600,7 +730,7 @@ void Smb4KPasswordHandler::slotGetPassword( const QString &username )
   if ( m_dlg && m_auth )
   {
     Smb4KAuthInfo *auth = readAuth( m_auth->workgroup().upper(), m_auth->host().upper(), username );
-    m_dlg->passwdEdit()->setText( auth->password() );
+    ((KLineEdit *)m_dlg->child( "AskPassPasswordEdit", "KLineEdit", true ))->setText( auth->password() );
     delete auth;
 
     m_auth->setShare( username );
@@ -608,6 +738,15 @@ void Smb4KPasswordHandler::slotGetPassword( const QString &username )
 }
 
 
+void Smb4KPasswordHandler::slotEnableOKButton( const QString &text )
+{
+  if ( m_dlg )
+  {
+    m_dlg->enableButtonOK( !text.isEmpty() );
+  }
+}
+
+
 void Smb4KPasswordHandler::slotReceivePassword( KProcess *, char *buffer, int buflen )
 {
 #ifdef __FreeBSD__
@@ -618,9 +757,11 @@ void Smb4KPasswordHandler::slotReceivePassword( KProcess *, char *buffer, int bu
 
 void Smb4KPasswordHandler::slotWritePassword( KProcess *proc )
 {
-#ifdef __FreeBSD__
+
   delete proc;
 
+#ifdef __FreeBSD__
+
   QString pass = m_buffer.remove( "\n" ).stripWhiteSpace();
   m_buffer = QString::null;
 
@@ -649,9 +790,9 @@ void Smb4KPasswordHandler::slotWritePassword( KProcess *proc )
   if ( contents.find( "[default]" ) == contents.end() &&
        contents.find( "[DEFAULT]" ) == contents.end() )
   {
-    QMap<QString,QString> map = readGlobalSMBOptions();
+    QMap<QString,QString> map = globalSMBOptions();
     QString workgroup = map["workgroup"];
-    QString wins = getWINSServer();
+    QString wins = winsServer();
 
     QStringList::Iterator it = contents.prepend( "[default]" );
     it++;
@@ -742,7 +883,7 @@ void Smb4KPasswordHandler::slotWritePassword( KProcess *proc )
   }
   else
   {
-    emit error( ERROR_WRITING_FILE, "~/.nsmbrc" );
+    showCoreError( ERROR_WRITING_FILE, "~/.nsmbrc" );
 
     return;
   }
@@ -750,114 +891,23 @@ void Smb4KPasswordHandler::slotWritePassword( KProcess *proc )
   // Get minimal security: Fix permissions.
   QString path = QDir::homeDirPath()+"/"+file.name();
 
-  if ( chmod( path.ascii(), 00600 ) == -1 )
+  int fd;
+
+  if ( (fd = open( path.ascii(), O_WRONLY )) == -1 )
   {
-    // FIXME: Do we need to emit an error here at all?
+    int err_no = errno;
+    showCoreError( ERROR_OPENING_FILE, strerror( err_no ) );
+
+    return;
+  }
+  else
+  {
+    if ( fchmod( fd, 00600 ) == -1 )
+    {
+      // FIXME: Do we need to emit an error here at all?
+    }
   }
 #endif
 }
 
-
-
-
-/***************************************************************************
- *                                                                         *
- *  The Smb4KPasswordHandler::AskPass class.                               *
- *                                                                         *
- ***************************************************************************/
-
-Smb4KPasswordHandler::AskPass::AskPass( Smb4KAuthInfo *auth, int desc, QWidget *parent, const char *name )
-: KDialogBase( Plain, i18n( "Authentication" ), Ok|Cancel, Ok, parent, name, false, true )
-{
-  // WARNING: Do not set the Qt::WDestructiveClose flag! You'll get a crash in the Smb4KPasswordHandler::askpass()
-  // function.
-
-  QFrame *frame = plainPage();
-  QGridLayout *layout = new QGridLayout( frame );
-  layout->setSpacing( 10 );
-  layout->setMargin( 0 );
-
-  QString message;
-
-  switch ( desc )
-  {
-    case NewData:
-      break;
-    case AccessDenied:
-      message = i18n( "The access was denied. " );
-      break;
-    case BadPassword:
-      message = i18n( "The password is not correct. " );
-      break;
-    case PermDenied:
-      message = i18n( "The permission was denied. " );
-      break;
-    case AuthError:
-      message = i18n( "An authentication error occurred. " );
-      break;
-    case LogonFailure:
-      message = i18n( "The logon failed. " );
-      break;
-    default:
-      break;
-  }
-
-  message.append( i18n( "Please enter authentication data for %1." ).arg( auth->share().stripWhiteSpace().isEmpty() ? auth->host() : "//"+auth->host()+"/"+auth->share() ) );
-
-  QGroupBox *message_box = new QGroupBox( 1, Qt::Horizontal, i18n( "Information" ), frame, "AskPassInfoBox" );
-  QLabel *message_label = new QLabel( message_box );
-  message_label->setText( message.stripWhiteSpace() );
-  message_label->setTextFormat( Qt::RichText );
-
-  layout->addWidget( message_box, 0, 0, 0 );
-
-  QGroupBox *input_box = new QGroupBox( 1, Qt::Horizontal, i18n( "Input" ), frame, "AskPassInputBox" );
-  QWidget *input_widget = new QWidget( input_box );
-  QGridLayout *input_widget_layout = new QGridLayout( input_widget );
-  input_widget_layout->setSpacing( 10 );
-  input_widget_layout->setMargin( 0 );
-
-  QLabel *user_label = new QLabel( i18n( "User:" ), input_widget );
-  input_widget_layout->addWidget( user_label, 0, 0, 0 );
-
-  m_user_edit = NULL;
-  m_user_combo = NULL;
-
-  if ( QString::compare( auth->share(), "homes" ) != 0 )
-  {
-    m_user_edit = new KLineEdit( input_widget, "AskPassUserEdit" );
-    m_user_edit->setMinimumWidth( 250 );
-    input_widget_layout->addWidget( m_user_edit, 0, 1, 0 );
-  }
-  else
-  {
-    m_user_combo = new KComboBox( input_widget, "AskPassUserCombo" );
-    m_user_combo->setEditable( true );
-    m_user_combo->setMinimumWidth( 250 );
-    input_widget_layout->addWidget( m_user_combo, 0, 1, 0 );
-  }
-
-  QLabel *password_label = new QLabel( i18n( "Password:" ), input_widget );
-  input_widget_layout->addWidget( password_label, 1, 0, 0 );
-
-  m_pass_edit = new KLineEdit( input_widget, "AskPassPasswordEdit" );
-  m_pass_edit->setEchoMode( KLineEdit::Password );
-  m_pass_edit->setMinimumWidth( 250 );
-  input_widget_layout->addWidget( m_pass_edit, 1, 1, 0 );
-
-  layout->addWidget( input_box, 1, 0, 0 );
-
-  message_box->setFixedWidth( input_box->sizeHint().width() );  // Do not change this! You'll get layout probems in the case you do.
-
-  setMainWidget( frame );
-
-  setFixedSize( sizeHint() );
-}
-
-
-Smb4KPasswordHandler::AskPass::~AskPass()
-{
-}
-
-
 #include "smb4kpasswordhandler.moc"
