@@ -3,16 +3,26 @@
                              -------------------
     begin                : Do Mr 4 2004
     copyright            : (C) 2004 by Franck Babin
+                           (C) 2005 by Alexander Reinholdt
     email                : babinfranck@yahoo.ca
+                           dustpuppy@mail.berlios.de
  ***************************************************************************/
 
 /***************************************************************************
- *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
+ *   This program is distributed in the hope that it will be useful, but   *
+ *   WITHOUT ANY WARRANTY; without even the implied warranty of            *
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
+ *   General Public License for more details.                              *
+ *                                                                         *
+ *   You should have received a copy of the GNU General Public License     *
+ *   along with this program; if not, write to the                         *
+ *   Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
+ *   MA  02111-1307 USA                                                    *
  ***************************************************************************/
 
 // Qt includes
@@ -21,15 +31,15 @@
 // system includes
 #include <unistd.h>
 #include <sys/types.h>
-  
+
 // application specific includes
 #include "smb4kshare.h"
 
- 
-Smb4KShare::Smb4KShare( const QString &name, const QString &path, const QString &filesystem, const int uid, const int gid, bool broken ) : m_name(name), m_path(path), m_filesystem( filesystem ), m_userinfo( uid, gid ), m_cifs_login( QString::null ), m_broken( broken )
+
+Smb4KShare::Smb4KShare( const QString &name, const QString &path, const QString &filesystem, const int uid, const int gid, bool broken ) : m_name(name), m_path(path), m_filesystem( filesystem ), m_userinfo( uid, gid ), m_cifs_login( QString::null ), m_broken( broken ), m_total( 0 ), m_free( 0 )
 {
   //FIXME should throw an exception if one of the param is empty
-  
+
   if ( uid != (int)getuid() && gid != (int)getgid() )
   {
     m_foreign_mount = true;
@@ -46,7 +56,7 @@ Smb4KShare::Smb4KShare( const QString &name, const QString &path, const QString
 }
 
 
-Smb4KShare::Smb4KShare( const Smb4KShare &s ) : m_name( s.getName() ), m_path( s.getPath() ), m_filesystem( s.getFilesystem() ), m_userinfo( s.getUID(), s.getGID() ), m_cifs_login( s.getCIFSLogin() ), m_foreign_mount( s.isForeign() ), m_broken( s.isBroken() )
+Smb4KShare::Smb4KShare( const Smb4KShare &s ) : m_name( s.name() ), m_path( s.path() ), m_filesystem( s.filesystem() ), m_userinfo( s.uid(), s.gid() ), m_cifs_login( s.cifsLogin() ), m_foreign_mount( s.isForeign() ), m_broken( s.isBroken() ), m_total( s.totalDiskSpace() ), m_free( s.freeDiskSpace() )
 {
 }
 
@@ -55,55 +65,55 @@ Smb4KShare::~Smb4KShare(){}
 
 
 
-QString Smb4KShare::getName() const
+QString Smb4KShare::name() const
 {
   return m_name;
 }
 
 
-QString Smb4KShare::getPath() const
+QString Smb4KShare::path() const
 {
   return m_path;
 }
 
 
-QString Smb4KShare::getCanonicalPath() const
+QString Smb4KShare::canonicalPath() const
 {
   return m_broken ? m_path : QDir( m_path ).canonicalPath();
 }
 
 
-int Smb4KShare::getUID() const
+int Smb4KShare::uid() const
 {
   return m_userinfo.uid();
 }
 
 
-int Smb4KShare::getGID() const
+int Smb4KShare::gid() const
 {
   return m_userinfo.gid();
 }
 
 
-QString Smb4KShare::getUser() const
+QString Smb4KShare::user() const
 {
   return m_userinfo.user();
 }
 
 
-QString Smb4KShare::getGroup() const
+QString Smb4KShare::group() const
 {
   return m_userinfo.group();
 }
 
 
-QString Smb4KShare::getFilesystem() const
+QString Smb4KShare::filesystem() const
 {
   return m_filesystem;
 }
 
 
-QString Smb4KShare::getCIFSLogin() const
+QString Smb4KShare::cifsLogin() const
 {
   return m_cifs_login;
 }
@@ -127,3 +137,31 @@ void Smb4KShare::setBroken( bool broken )
 }
 
 
+void Smb4KShare::setTotalDiskSpace( double total )
+{
+  m_total = total;
+}
+
+
+void Smb4KShare::setFreeDiskSpace( double free )
+{
+  m_free = free;
+}
+
+
+double Smb4KShare::totalDiskSpace() const
+{
+  return m_total;
+}
+
+
+double Smb4KShare::freeDiskSpace() const
+{
+  return m_free;
+}
+
+
+double Smb4KShare::percentage() const
+{
+  return (m_total - m_free) / m_total * 100;
+}
