@@ -36,14 +36,13 @@
 #include "core/smb4kdefs.h"
 #include "core/smb4kglobal.h"
 
-using namespace Smb4K_Global;
+using namespace Smb4KGlobal;
 
 #define ID_STATUS_MSG 1
 
 
 Smb4KApp::Smb4KApp( QWidget* , const char* name ) : KDockMainWindow( 0, name )
 {
-  m_config = kapp->config();
   m_startDocked = false;
 
   // The caption.
@@ -58,6 +57,12 @@ Smb4KApp::Smb4KApp( QWidget* , const char* name ) : KDockMainWindow( 0, name )
   setupMenuBar();
   setupSystemTray();
 
+  // Resize the main window. It'll start with a resolution
+  // of 800x600 pixels the first time. Later, this will be
+  // overwritten by readOptions().
+  resize( 800, 600 );
+
+  // Read the options for the main window
   readOptions();
 }
 
@@ -73,49 +78,57 @@ void Smb4KApp::initActions()
 
   quitApp = KStdAction::quit( this, SLOT( slotQuit() ), actionCollection(), "quit_smb4k" );
 
-  viewToolBar = KStdAction::showToolbar(this, SLOT(slotViewToolBar()), actionCollection());
-  viewStatusBar = KStdAction::showStatusbar(this, SLOT(slotViewStatusBar()), actionCollection());
+  setStandardToolBarMenuEnabled( true );
+  createStandardStatusBarAction();
 
   settingsSmb4KOptions = KStdAction::preferences( this, SLOT( slotSmb4KOptionsDlg() ), actionCollection(), "configure_smb4k" );
 
-  actionCollection()->insert( m_network_view->action_collection()->action( "abort" ) );
-  abort = m_network_view->action_collection()->action( "abort" );
+  abort = new KAction( i18n( "&Abort" ) , "stop", CTRL+Key_A, this, SLOT( slotAbort() ), actionCollection(), "abort" );
 
-  actionCollection()->insert( m_network_view->action_collection()->action( "network_rescan" ) );
-  networkRescan = m_network_view->action_collection()->action( "network_rescan" );
+  actionCollection()->insert( m_network_view->action_collection()->action( "rescan_action" ) );
+  networkRescan = m_network_view->action_collection()->action( "rescan_action" );
 
-  actionCollection()->insert( m_network_view->action_collection()->action("network_search" ) );
-  networkSearch = m_network_view->action_collection()->action("network_search" );
+  actionCollection()->insert( m_network_view->action_collection()->action( "search_action" ) );
+  networkSearch = m_network_view->action_collection()->action( "search_action" );
 
-  actionCollection()->insert( m_network_view->action_collection()->action( "network_preview" ) );
-  networkPreview = m_network_view->action_collection()->action( "network_preview" );
+  actionCollection()->insert( m_network_view->action_collection()->action( "preview_action" ) );
+  networkPreview = m_network_view->action_collection()->action( "preview_action" );
 
-  actionCollection()->insert( m_network_view->action_collection()->action( "network_askpass" ) );
-  networkAskPass = m_network_view->action_collection()->action( "network_askpass" );
+  actionCollection()->insert( m_network_view->action_collection()->action( "askpass_action" ) );
+  networkAskPass = m_network_view->action_collection()->action( "askpass_action" );
 
-  actionCollection()->insert( m_network_view->action_collection()->action( "network_print" ) );
-  networkPrint = m_network_view->action_collection()->action( "network_print" );
+  actionCollection()->insert( m_network_view->action_collection()->action( "print_action" ) );
+  networkPrint = m_network_view->action_collection()->action( "print_action" );
 
-  actionCollection()->insert( m_network_view->action_collection()->action( "network_mount" ) );
-  networkMount = m_network_view->action_collection()->action( "network_mount" );
+  actionCollection()->insert( m_network_view->action_collection()->action( "mount_action" ) );
+  networkMount = m_network_view->action_collection()->action( "mount_action" );
 
-  actionCollection()->insert( m_network_view->action_collection()->action( "network_add_bookmark" ) );
-  networkAddBookmark = m_network_view->action_collection()->action( "network_add_bookmark" );
+  actionCollection()->insert( m_network_view->action_collection()->action( "add_bookmark_action" ) );
+  networkAddBookmark = m_network_view->action_collection()->action( "add_bookmark_action" );
 
-  actionCollection()->insert( m_share_view->action_collection()->action( "shares_unmount_share" ) );
-  sharesUnmount = m_share_view->action_collection()->action( "shares_unmount_share" );
+  actionCollection()->insert( m_network_view->action_collection()->action( "manual_mount_action" ) );
+  sharesManualMount = m_share_view->action_collection()->action( "manual_mount_action" );
 
-  actionCollection()->insert( m_share_view->action_collection()->action( "shares_force_unmount" ) );
-  sharesForceUnmount = m_share_view->action_collection()->action( "shares_force_unmount" );
+  actionCollection()->insert( m_share_view->action_collection()->action( "unmount_action" ) );
+  sharesUnmount = m_share_view->action_collection()->action( "unmount_action" );
 
-  actionCollection()->insert( m_share_view->action_collection()->action( "shares_unmount_all" ) );
-  sharesUnmountAll = m_share_view->action_collection()->action( "shares_unmount_all" );
+  actionCollection()->insert( m_share_view->action_collection()->action( "force_unmount_action" ) );
+  sharesForceUnmount = m_share_view->action_collection()->action( "force_unmount_action" );
 
-  actionCollection()->insert( m_share_view->action_collection()->action( "shares_manual_mount" ) );
-  sharesManualMount = m_share_view->action_collection()->action( "shares_manual_mount" );
+  actionCollection()->insert( m_share_view->action_collection()->action( "unmount_all_action" ) );
+  sharesUnmountAll = m_share_view->action_collection()->action( "unmount_all_action" );
 
-  actionCollection()->insert( m_share_view->action_collection()->action( "shares_start_filemanager" ) );
-  sharesStartFilemanager = m_share_view->action_collection()->action( "shares_start_filemanager" );
+  actionCollection()->insert( m_share_view->action_collection()->action( "filemanager_action" ) );
+  sharesStartFilemanager = m_share_view->action_collection()->action( "filemanager_action" );
+
+  actionCollection()->insert( m_share_view->action_collection()->action( "synchronize_action" ) );
+  sharesSynchronize = m_share_view->action_collection()->action( "synchronize_action" );
+
+  // We need some KAccelActions to jump to the dock widgets.
+  accel()->insert( i18n( "Jump to Network Browser" ), CTRL+Key_1, this, SLOT( slotShowNetworkBrowser() ), false, true );
+  accel()->insert( i18n( "Jump to Search Dialog" ), CTRL+Key_2, this, SLOT( slotShowSearchDialog() ), false, true );
+//   accel()->insert( i18n( "Jump to Search Dialog" ), CTRL+Key_S, this, SLOT( slotShowSearchDialog() ), false, true );
+  accel()->insert( i18n( "Jump to Shares View" ), CTRL+Key_3, this, SLOT( slotJumpToSharesView() ), false, true );
 
   // use the absolute path to your smb4kui.rc file for testing purpose in createGUI();
   createGUI();
@@ -142,7 +155,7 @@ void Smb4KApp::initStatusBar()
   m_progress->setPercentageVisible( false );
 
   m_progTimer = new QTimer( this );
-  connect( m_progTimer, SIGNAL( timeout() ), SLOT( slotSetProgress() ) );
+  connect( m_progTimer, SIGNAL( timeout() ), this, SLOT( slotSetProgress() ) );
 
   statusBar()->addWidget( m_winsLabel, false );
   statusBar()->addWidget( m_listLabel, false );
@@ -171,35 +184,22 @@ void Smb4KApp::initView()
   setView( main );
   setMainDockWidget( main );
 
-  KDockWidget *network = createDockWidget( "NetworkView", SmallIcon( "network" ), 0L, i18n( "Network Browser" ) );
+  KDockWidget *network = createDockWidget( "NetworkBrowser", SmallIcon( "network" ), 0L, i18n( "Network Browser" ) );
   m_network_view = new Smb4KBrowserWidget( network );
   network->setWidget( m_network_view );
-  network->manualDock( main, KDockWidget::DockLeft, 30 );
+  network->manualDock( main, KDockWidget::DockLeft, 40 );
 
-  KDockWidget *tabs = createDockWidget( "TabsView", SmallIcon( "tab_new_raised" ), 0L, i18n( "Tab Widget" ) );
-  KDockTabGroup *info = new KDockTabGroup( tabs, "TabGroup" );
-  tabs->setWidget( info );
-  tabs->manualDock( main, KDockWidget::DockBottom, 80 );
+  KDockWidget *search = createDockWidget( "SearchDialog", SmallIcon( "find" ), 0L, i18n( "Search Dialog" ) );
+  m_search_dialog = new Smb4KSearchDialog( search );
+  search->setWidget( m_search_dialog );
+  search->manualDock( network, KDockWidget::DockCenter, 0 );
 
-  m_network_tab = new Smb4KNetworkTab( info );
-  info->insertTab( m_network_tab, QIconSet( SmallIcon( "network" ), QIconSet::Automatic ), i18n( "Network" ), 0 );
-
-  m_share_tab = new Smb4KShareTab( info );
-  info->insertTab( m_share_tab, QIconSet( SmallIcon( "hdd_mount" ), QIconSet::Automatic ), i18n( "Share" ), 1 );
-
-  m_search_tab = new Smb4KSearchTab( info );
-  info->insertTab( m_search_tab, QIconSet( SmallIcon( "find" ), QIconSet::Automatic ), i18n( "Search" ), 2 );
-
-  connect( m_network_view, SIGNAL( searchRequest() ), this,                             SLOT( slotShowSearchTab() ) );
-  connect( m_network_view, SIGNAL( highlightedItem( QListViewItem * ) ), m_network_tab, SLOT( slotGetListViewItem( QListViewItem * ) ) );
-  connect( m_network_view, SIGNAL( cleared() ), m_network_tab,                          SLOT( slotClearWidget() ) );
-  connect( m_share_view,   SIGNAL( cleared() ), m_share_tab,                            SLOT( slotClearWidget() ) );
-  connect( m_share_view,   SIGNAL( clearedSelection() ), m_share_tab,                   SLOT( slotClearWidget() ) );
-  connect( m_search_tab,   SIGNAL( searchResult(Smb4KHostItem* ) ), m_network_view,     SLOT( slotInsertItem(Smb4KHostItem* ) ) );
-
-  accel()->insert( i18n( "Show network tab" ), CTRL+Key_1, this, SLOT( slotShowNetworkTab() ), false, true );
-  accel()->insert( i18n( "Show Share tab" ),   CTRL+Key_2, this, SLOT( slotShowShareTab() ),   false, true );
-  accel()->insert( i18n( "Show Search tab" ),  CTRL+Key_3, this, SLOT( slotShowSearchTab() ),  false, true );
+  connect( m_network_view,  SIGNAL( searchRequest() ),
+           this,            SLOT( slotShowSearchDialog() ) );
+  connect( m_search_dialog, SIGNAL( searchResult( Smb4KHostItem * ) ),
+           m_network_view,  SLOT( slotInsertItem( Smb4KHostItem * ) ) );
+  connect( m_network_view,  SIGNAL( itemInserted( Smb4KHostItem * ) ),
+           m_search_dialog, SLOT( slotItemAddedToBrowser( Smb4KHostItem * ) ) );
 }
 
 
@@ -223,59 +223,59 @@ void Smb4KApp::setupSystemTray()
 
 void Smb4KApp::saveOptions()
 {
-  m_config->setGroup("General Options");
-  m_config->writeEntry("Show Toolbar", viewToolBar->isChecked());
-  m_config->writeEntry("Show Statusbar",viewStatusBar->isChecked());
-  m_config->writeEntry("Toolbar Position", (int)toolBar("mainToolBar")->barPos());
-  m_config->writeEntry("Toolbar Text Position", toolBar("mainToolBar")->iconText());
-  m_config->writeEntry("Toolbar Icon Size", toolBar("mainToolBar")->iconSize());
+  saveMainWindowSettings( config(), "Main Window" );
+  writeDockConfig( config(), "Main Window" );
 
-  writeDockConfig( m_config, "General Options" );
-
-  KConfigGroup settingsConfig( KGlobal::config(), "Settings" );
-  settingsConfig.writeEntry( "StartDocked", m_startDocked );
+  config()->setGroup( "Docking" );
+  config()->writeEntry( "Start Docked", m_startDocked );
 }
 
 
 void Smb4KApp::readOptions()
 {
-  m_config->setGroup("General Options");
+  applyMainWindowSettings( config(), "Main Window" );
+  readDockConfig( config(), "Main Window" );
 
-  // bar status settings
-  bool bViewToolbar = m_config->readBoolEntry("Show Toolbar", true);
-  viewToolBar->setChecked(bViewToolbar);
-  slotViewToolBar();
+  // We have to set the tab labels here. The writeDockConfig()
+  // readDockConfig() combo will ignore changes of the localization,
+  // because the first labels will be saved and then loaded again
+  // and again.
+  KDockWidget *network_view = manager()->getDockWidgetFromName( "NetworkBrowser" );
+  network_view->setTabPageLabel( i18n( "Network Browser" ) );
 
-  bool bViewStatusbar = m_config->readBoolEntry("Show Statusbar", true);
-  viewStatusBar->setChecked(bViewStatusbar);
-  slotViewStatusBar();
+  KDockWidget *search_dialog = manager()->getDockWidgetFromName( "SearchDialog" );
+  search_dialog->setTabPageLabel( i18n( "Search Dialog" ) );
 
+  KDockTabGroup *network_parent = NULL;
+  KDockTabGroup *search_parent = NULL;
 
-  // bar position settings
-  KToolBar::BarPosition toolBarPos;
-  toolBarPos=(KToolBar::BarPosition) m_config->readNumEntry("Toolbar Position", KToolBar::Top);
-  toolBar("mainToolBar")->setBarPos(toolBarPos);
+  if ( (network_parent = network_view->parentDockTabGroup()) )
+  {
+    network_parent->changeTab( network_view, i18n( "Network Browser" ) );
+  }
 
-  KToolBar::IconText toolBarIconText;
-  toolBarIconText=(KToolBar::IconText) m_config->readNumEntry("Toolbar Text Position", KToolBar::IconOnly);
-  toolBar("mainToolBar")->setIconText(toolBarIconText);
-  toolBar("mainToolBar")->setIconSize( m_config->readNumEntry( "Toolbar Icon Size", 0 ) );
-
-  readDockConfig( m_config, "General Options" );
+  if ( (search_parent = search_dialog->parentDockTabGroup()) )
+  {
+    search_parent->changeTab( search_dialog,  i18n( "Search Dialog" ) );
+  }
 
   // Set-up the status bar.
-  m_config->setGroup( "Browse Options" );
-  
-  if ( QString::compare( m_config->readEntry( "Browse List", "nmblookup" ), "nmblookup" ) == 0 )
+  config()->setGroup( "Browse Options" );
+
+  if ( QString::compare( config()->readEntry( "Browse List", "lookup" ), "lookup" ) == 0 )
   {
     m_listLabel->setText( i18n( "Network" ) );
   }
+  else if ( QString::compare( config()->readEntry( "Browse List", "lookup" ), "master" ) == 0 )
+  {
+    m_listLabel->setText( i18n( "Master" ) );
+  }
   else
   {
-    m_listLabel->setText( m_config->readEntry( "Query Host" ) );
+    m_listLabel->setText( config()->readEntry( "Query Host" ) );
   }
-    
-  QString wins = getWINSServer();
+
+  QString wins = winsServer();
 
   if ( QString::compare( wins, QString::null ) != 0 )
   {
@@ -359,18 +359,13 @@ void Smb4KApp::slotAbort()
 
 void Smb4KApp::slotCoreRunStateChanged()
 {
-  int mode = smb4k_core->currentState();
+  int state = smb4k_core->currentState();
 
   if ( smb4k_core->isRunning() )
   {
     abort->setEnabled( true );
 
-    if ( smb4k_core->scannerIsRunning() )
-      networkRescan->setEnabled( false );
-    else
-      networkRescan->setEnabled( true );
-
-    switch ( mode )
+    switch ( state )
     {
       case SCANNER_INIT:
         slotStatusMsg( i18n( "Searching network..." ) );
@@ -381,6 +376,9 @@ void Smb4KApp::slotCoreRunStateChanged()
       case SCANNER_OPENING_HOST:
         slotStatusMsg( i18n( "Opening host..." ) );
         break;
+      case SCANNER_RETRYING_OPENING_HOST:
+        slotStatusMsg( i18n( "Retrying to open host..." ) );
+        break;
       case SCANNER_RETRIEVING_INFO:
         slotStatusMsg( i18n( "Retrieving information..." ) );
         break;
@@ -390,9 +388,6 @@ void Smb4KApp::slotCoreRunStateChanged()
       case SCANNER_SEARCHING:
         slotStatusMsg( i18n( "Searching for host..." ) );
         break;
-      case IPFINDER_WORKING:
-        slotStatusMsg( i18n( "Looking for IP addresses..." ) );
-        break;
       case MOUNTER_MOUNTING:
         slotStatusMsg( i18n( "Mounting share..." ) );
         break;
@@ -408,6 +403,9 @@ void Smb4KApp::slotCoreRunStateChanged()
       case PRINT_CONVERT_FILE:
         slotStatusMsg( i18n( "Converting file to Postscript..." ) );
         break;
+      case SYNCER_START:
+        slotStatusMsg( i18n( "Synchronizing data..." ) );
+        break;
       default:
         break;
     }
@@ -424,7 +422,6 @@ void Smb4KApp::slotCoreRunStateChanged()
     m_progTimer->stop();
     m_progress->setTotalSteps( 100 );
     m_progress->reset();
-    networkRescan->setEnabled( true );
     abort->setEnabled( false );
     slotStatusMsg( i18n( "Done." ) );
   }
@@ -436,43 +433,12 @@ void Smb4KApp::slotSmb4KOptionsDlg()
   if ( child( "ConfigDialog", "KDialogBase", true ) == 0 )
   {
     Smb4KOptionsDlg *configDlg = new Smb4KOptionsDlg( this, "ConfigDialog", false );
-    connect( configDlg, SIGNAL( read() ), this, SLOT( slotReadOptions() ) );
+    connect( configDlg, SIGNAL( savedOptions() ), this, SLOT( slotReadOptions() ) );
     configDlg->show();
   }
 }
 
 
-
-void Smb4KApp::slotViewToolBar()
-{
-  ///////////////////////////////////////////////////////////////////
-  // turn Toolbar on or off
-  if(!viewToolBar->isChecked())
-  {
-    toolBar("mainToolBar")->hide();
-  }
-  else
-  {
-    toolBar("mainToolBar")->show();
-  }
-}
-
-
-void Smb4KApp::slotViewStatusBar()
-{
-  ///////////////////////////////////////////////////////////////////
-  //turn Statusbar on or off
-  if(!viewStatusBar->isChecked())
-  {
-    statusBar()->hide();
-  }
-  else
-  {
-    statusBar()->show();
-  }
-}
-
-
 void Smb4KApp::slotStatusMsg(const QString &text)
 {
   ///////////////////////////////////////////////////////////////////
@@ -490,22 +456,30 @@ void Smb4KApp::slotSetProgress()
 
 void Smb4KApp::slotReadOptions()
 {
+  // NOTE: You can put the readOptions() functione here,
+  // but then there will be some unwanted flickering. This
+  // is the better way.
+
   m_network_view->readOptions();
   m_share_view->readOptions();
 
   // Set-up the status bar.
-  m_config->setGroup( "Browse Options" );
-  
-  if ( QString::compare( m_config->readEntry( "Browse List", "nmblookup" ), "nmblookup" ) == 0 )
+  config()->setGroup( "Browse Options" );
+
+  if ( QString::compare( config()->readEntry( "Browse List", "lookup" ), "lookup" ) == 0 )
   {
     m_listLabel->setText( i18n( "Network" ) );
   }
+  else if ( QString::compare( config()->readEntry( "Browse List", "lookup" ), "master" ) == 0 )
+  {
+    m_listLabel->setText( i18n( "Master" ) );
+  }
   else
   {
-    m_listLabel->setText( m_config->readEntry( "Query Host" ) );
+    m_listLabel->setText( config()->readEntry( "Query Host" ) );
   }
-  
-  QString wins = getWINSServer();
+
+  QString wins = winsServer();
 
   if ( QString::compare( wins, QString::null ) != 0 )
   {
@@ -521,30 +495,31 @@ void Smb4KApp::slotReadOptions()
 }
 
 
-void Smb4KApp::slotShowNetworkTab()
+void Smb4KApp::slotShowSearchDialog()
 {
-  if ( (KDockTabGroup *)child( "TabGroup", "KDockTabGroup", true ) && ((KDockTabGroup *)child( "TabGroup", "KDockTabGroup", true ))->isShown() )
+  if ( manager()->getDockWidgetFromName( "SearchDialog" )->isHidden() )
   {
-    ((KDockTabGroup *)child( "TabGroup", "KDockTabGroup", true ))->showPage( m_network_tab );
+    manager()->getDockWidgetFromName( "SearchDialog" )->changeHideShowState();
   }
+
+  m_search_dialog->comboBox()->lineEdit()->setFocus();
 }
 
 
-void Smb4KApp::slotShowShareTab()
+void Smb4KApp::slotShowNetworkBrowser()
 {
-  if ( (KDockTabGroup *)child( "TabGroup", "KDockTabGroup", true ) && ((KDockTabGroup *)child( "TabGroup", "KDockTabGroup", true ))->isShown() )
+  if ( manager()->getDockWidgetFromName( "NetworkBrowser" )->isHidden() )
   {
-    ((KDockTabGroup *)child( "TabGroup", "KDockTabGroup", true ))->showPage( m_share_tab );
+    manager()->getDockWidgetFromName( "NetworkBrowser" )->changeHideShowState();
   }
+
+  m_network_view->setFocus();
 }
 
 
-void Smb4KApp::slotShowSearchTab()
+void Smb4KApp::slotJumpToSharesView()
 {
-  if ( (KDockTabGroup *)child( "TabGroup", "KDockTabGroup", true ) && ((KDockTabGroup *)child( "TabGroup", "KDockTabGroup", true ))->isShown() )
-  {
-    ((KDockTabGroup *)child( "TabGroup", "KDockTabGroup", true ))->showPage( m_search_tab );
-  }
+  m_share_view->setFocus();
 }
 
 #include "smb4k.moc"
