@@ -40,6 +40,7 @@
 #include <kdialogbase.h>
 #include <kcombobox.h>
 #include <ktoolbarbutton.h>
+#include <kaction.h>
 
 // plugin specific includes
 // #include "smb4k_konqplugin.moc"
@@ -48,7 +49,7 @@
 #include "widgets/smb4koptionsdlg.h"
 #include "core/smb4kglobal.h"
 
-using namespace  Smb4K_Global;
+using namespace  Smb4KGlobal;
 
 
 KonqSidebar_Smb4K::KonqSidebar_Smb4K(KInstance *inst,QObject *parent,QWidget *widgetParent, QString &desktopName, const char* name):
@@ -94,43 +95,70 @@ KonqSidebar_Smb4K::KonqSidebar_Smb4K(KInstance *inst,QObject *parent,QWidget *wi
 
      // The core:
      m_core = new Smb4KCore( widgetParent, "Core" );
-     m_menu = new KPopupMenu( widgetParent );
+
+     // Action menu stuff
+     m_collection = new KActionCollection( this, "KonquerorPlugin_ActionCollection", KGlobal::instance() );
+     m_menu = new Smb4KBrowserActionMenu( Smb4KBrowserActionMenu::KonqPlugin, m_collection, QString::null, QIconSet(), widget, "KonquerorPlugin_ActionMenu" );
+     m_menu->popupMenu()->insertSeparator( -1 );
+     m_menu->insert( new KAction( i18n( "Configuration" ), "configure", CTRL+Key_O, 0, 0, m_collection, "configure_action" ) );
+
+     m_menu->askpassAction()->setEnabled( false );
+     m_menu->mountAction()->setEnabled( false );
 
      slotReadOptions();
 
-     // Connections
-     connect( m_core, SIGNAL( runStateChanged() ), this, SLOT( slotCoreRunStateChanged() ) );
+     // Action connections
+     connect( m_collection->action( "rescan_action" ),    SIGNAL( activated() ), this, SLOT( slotRescan() ) );
+     connect( m_collection->action( "askpass_action" ),   SIGNAL( activated() ), this, SLOT( slotAskPass() ) );
+     connect( m_collection->action( "mount_action" ),     SIGNAL( activated() ), this, SLOT( slotMountShare() ) );
+     connect( m_collection->action( "configure_action" ), SIGNAL( activated() ), this, SLOT( slotSmb4KOptionsDlg() ) );
 
      // Widget specific connections.
      connect( widget, SIGNAL( executed( QListViewItem * ) ), SLOT( slotItemExecuted( QListViewItem * ) ) );
      connect( widget, SIGNAL( expanded( QListViewItem * ) ), SLOT( slotItemExpandedCollapsed( QListViewItem * ) ) );
      connect( widget, SIGNAL( collapsed( QListViewItem * ) ), SLOT( slotItemExpandedCollapsed( QListViewItem * ) ) );
      connect( widget, SIGNAL( selectionChanged( QListViewItem * ) ), SLOT( slotSelectionChanged( QListViewItem * ) ) );
-     connect( widget, SIGNAL( clicked( QListViewItem * ) ), SLOT( slotItemClicked( QListViewItem * ) ) );
      connect( widget, SIGNAL( rightButtonPressed( QListViewItem *, const QPoint &, int ) ), SLOT( slotRightButtonPressed( QListViewItem *, const QPoint &, int ) ) );
-     connect( widget, SIGNAL( setOpen( QListViewItem *, bool ) ), SLOT( slotSetOpen( QListViewItem *, bool ) ) );
 
      // External connections.
-
+     connect( m_core,            SIGNAL( runStateChanged() ), this, SLOT( slotCoreRunStateChanged() ) );
      connect( m_core->scanner(), SIGNAL( workgroups( const QValueList<Smb4KWorkgroupItem *> & ) ), this, SLOT( slotWorkgroups( const QValueList<Smb4KWorkgroupItem *> & ) ) );
      connect( m_core->scanner(), SIGNAL( members( const QValueList<Smb4KHostItem *> & ) ), this, SLOT( slotMembers( const QValueList<Smb4KHostItem *> & ) ) );
      connect( m_core->scanner(), SIGNAL( shares( const QValueList<Smb4KShareItem *> & ) ), this, SLOT( slotShares( const QValueList<Smb4KShareItem *> & ) ) );
-     connect( m_core->ipFinder(), SIGNAL( ip_address( Smb4KHostItem * ) ), this, SLOT( slotAddIPAddress( Smb4KHostItem * ) ) );
-     connect( m_core->mounter(),   SIGNAL( updated() ), this, SLOT( slotMarkShares() ) );
+     connect( m_core->scanner(), SIGNAL( ipAddress( Smb4KHostItem * ) ), this, SLOT( slotAddIPAddress( Smb4KHostItem * ) ) );
+     connect( m_core->mounter(), SIGNAL( updated() ), this, SLOT( slotMarkShares() ) );
      connect( m_core->mounter(), SIGNAL( mountedShare( const QString &) ), this, SLOT( slotMountedShare( const QString & ) ) );
+     connect( m_core->mounter(), SIGNAL( alreadyMountedShare( const QString &) ), this, SLOT( slotMountedShare( const QString & ) ) );
 }
 
 
 KonqSidebar_Smb4K::~KonqSidebar_Smb4K()
 {
- m_core->mounter()->unmountAllShares();
+  if ( m_menu )
+  {
+    delete m_menu;
+  }
+
+  uint index = 0;
+
+  while ( index < m_collection->count() )
+  {
+    delete m_collection->action( index++ );
+  }
+
+  m_collection->clear();
+
+  // FIXME: This definitely does not work!
+  // Massimo: Is there a possibility to use the KApplication::shutDown() signal
+  // or similar?
+  m_core->mounter()->unmountAllShares();
 }
 
 extern "C"
 {
     void* create_konqsidebar_smb4k(KInstance *instance,QObject *par,QWidget *widp,QString &desktopname,const char *name)
     {
-     KGlobal::locale()->insertCatalogue("konqsidebar_smb4k");
+     KGlobal::locale()->insertCatalogue( "smb4k" );
      return new KonqSidebar_Smb4K(instance,par,widp,desktopname,name);
     }
 };
@@ -155,19 +183,53 @@ void KonqSidebar_Smb4K::handleURL(const KURL &url)
  //kdDebug() << "KonqSidebar_Smb4K::handleURL - "<< currentKonquerorURL << endl;
 }
 
-void KonqSidebar_Smb4K::slotSetOpen( QListViewItem *item, bool open )
+void KonqSidebar_Smb4K::setOpen( QListViewItem *item, bool open, bool use_setOpen )
 {
-  if ( open && item->depth() == 0 )
+  if ( open )
   {
-    Smb4KWorkgroupItem *i = m_core->scanner()->getWorkgroup( item->text( Network ) );
+    switch ( item->depth() )
+    {
+      case 0:
+      {
+        Smb4KWorkgroupItem *i = smb4k_core->scanner()->getWorkgroup( item->text( Network ) );
+        smb4k_core->scanner()->getWorkgroupMembers( item->text( Network ), i->master(), i->ip() );
+        break;
+      }
+      case 1:
+      {
+        smb4k_core->scanner()->getShares( item->parent()->text( Network ), item->text( Network ), item->text( IP ) );
+        break;
+      }
+      default:
+        break;
+    }
+  }
+  else
+  {
+    switch ( item->depth() )
+    {
+      case 1:
+      {
+        // Remove all children (shares).
+        QListViewItem *shareItem = item->firstChild();
 
-    m_core->scanner()->getWorkgroupMembers( item->text( Network ), i->master(), i->ip() );
+        while ( shareItem )
+        {
+          delete shareItem;
+          shareItem = item->firstChild();
+        }
+
+        break;
+      }
+      default:
+        break;
+    }
   }
-  else if ( open && item->depth() == 1 && item->childCount() == 0 )
+
+  if ( use_setOpen )
   {
-    m_core->scanner()->getShares( item->parent()->text( Network ), item->text( Network ), item->text( IP ) );
+    widget->setOpen( item, open );
   }
-  widget->setOpen( item, open );
 }
 
 void KonqSidebar_Smb4K::slotCoreRunStateChanged()
@@ -177,18 +239,23 @@ void KonqSidebar_Smb4K::slotCoreRunStateChanged()
 
 void KonqSidebar_Smb4K::slotItemExecuted( QListViewItem *item )
 {
-  if ( item && item->depth() == 2 )
+  if ( item )
   {
-    if ( item->text( Type ).contains( "Printer" ) == 0 )
+    if ( item->depth() == 2 )
     {
-      slotMountShare();
+      if ( item->text( Type ).contains( "Printer" ) == 0 )
+      {
+        slotMountShare();
+      }
+//       else
+//       {
+//         slotPrint();
+//       }
     }
-/*
     else
     {
-      slotPrint();
+      setOpen( item, !item->isOpen(), true );
     }
-*/
   }
 }
 
@@ -196,119 +263,67 @@ void KonqSidebar_Smb4K::slotItemExecuted( QListViewItem *item )
 void KonqSidebar_Smb4K::slotItemExpandedCollapsed( QListViewItem *item )
 {
   widget->setSelected( item, true );
+
+  // Since it seems to be not easy to intercept KListView::setOpen(), we
+  // handle the opening and closing of an item via the root decoration
+  // this way:
+  setOpen( item, item->isOpen(), false );
 }
 
 
 void KonqSidebar_Smb4K::slotSelectionChanged( QListViewItem *item )
 {
- bool open = true;
-
- if ( open && item->depth() == 0 )
+  if ( item && item->depth() == 2 )
   {
-    Smb4KWorkgroupItem *i = m_core->scanner()->getWorkgroup( item->text( Network ) );
-
-    m_core->scanner()->getWorkgroupMembers( item->text( Network ), i->master(), i->ip() );
-  }
-  else if ( open && item->depth() == 1 && item->childCount() == 0 )
-  {
-    m_core->scanner()->getShares( item->parent()->text( Network ), item->text( Network ), item->text( IP ) );
-  }
-  widget->setOpen( item, open );
- return;
-}
-
-
-void KonqSidebar_Smb4K::slotRightButtonPressed( QListViewItem *item, const QPoint &point, int col )
-{
-  m_menu->clear();
-
-  // The icon sets:
-  QIconSet reloadSet( SmallIcon( "reload" ), QIconSet::Automatic );
-  QIconSet abortSet( SmallIcon( "stop" ), QIconSet::Automatic );
-  QIconSet searchSet( SmallIcon( "find" ), QIconSet::Automatic );
-  QIconSet authSet( SmallIcon( "personal" ), QIconSet::Automatic );
-  //QIconSet previewSet( SmallIcon( "view_icon" ), QIconSet::Automatic );
-  //QIconSet printSet( SmallIcon( "printer1" ), QIconSet::Automatic );
-  QIconSet mountSet( SmallIcon( "hdd_mount" ), QIconSet::Automatic );
-  QIconSet unmountSet( SmallIcon( "hdd_unmount" ), QIconSet::Automatic );
-  QIconSet prefSet( SmallIcon( "configure" ), QIconSet::Automatic );
-
-  // Menu items
-  m_menu->insertTitle( SmallIcon( "network" ), i18n( "Network" ), Title, 0 );
-  m_menu->insertItem( reloadSet, i18n( "Rescan" ), m_core->scanner(), SLOT( slotInit() ), CTRL+Key_R, Rescan, 1 );
-  m_menu->insertSeparator( 2 );
-  m_menu->insertItem( authSet, i18n( "Authentication" ), this, SLOT( slotAskPass() ), CTRL+Key_T, Password, 7 );
-  //m_menu->insertItem( previewSet, i18n( "Preview" ), this, SLOT( slotPreview() ), CTRL+Key_V, Preview, 8 );
-  //m_menu->insertItem( printSet, i18n( "Print File" ), this, SLOT( slotPrint() ), CTRL+Key_P, Print, 9 );
-
-  if ( item )
-  {
-    if(((Smb4KBrowserWidgetItem *)item)->isMounted())
-      m_menu->insertItem( unmountSet, i18n( "Unmount" ), this, SLOT( slotUnmountShare() ), CTRL+Key_U, Mount, 10 );
-    else
-      m_menu->insertItem( mountSet, i18n( "Mount" ), this, SLOT( slotMountShare() ), CTRL+Key_M, Mount, 10 );
-  }
-  // Maybe you want to remove the following since it is not necessary to put this item
-  // into the menu.
-  else
-  {
-    m_menu->insertItem( mountSet, i18n( "Mount" ), this, SLOT( slotMountShare() ), CTRL+Key_M, Mount, 10 );
-  }
-
-  m_menu->insertSeparator( 5 );
-  m_menu->insertItem( prefSet, i18n( "Configuration" ), this, SLOT( slotSmb4KOptionsDlg() ), CTRL+Key_O, Preferences, 10 );
-
-  // Enable and disable menu items.
-  if ( m_core->scannerIsRunning() || m_core->mounterIsRunning() )
-  {
-    m_menu->setItemEnabled( Abort, true );
-
-    if ( m_core->scannerIsRunning() && m_core->scannerState() != SCANNER_STOP )
-    {
-      m_menu->setItemEnabled( Rescan, false );
-    }
-    else
-      m_menu->setItemEnabled( Rescan, true );
-  }
-  else
-  {
-    m_menu->setItemEnabled( Rescan, true );
-    m_menu->setItemEnabled( Abort, false );
-  }
-
-  if ( item && item->depth() == 2 && col < widget->columns() )
-  {
-    m_menu->setItemEnabled( Password, true );
+    m_collection->action( "askpass_action" )->setEnabled( true );
 
     if ( item->text( Type ).contains( "Printer" ) == 0 )
     {
-      //m_menu->setItemEnabled( Preview, true );
-      //m_menu->setItemEnabled( Print, false );
-      m_menu->setItemEnabled( Mount, true );
+      m_collection->action( "mount_action" )->setEnabled( true );
     }
     else
     {
-      //m_menu->setItemEnabled( Preview, false );
-      //m_menu->setItemEnabled( Print, true );
-      m_menu->setItemEnabled( Mount, false );
+      m_collection->action( "mount_action" )->setEnabled( false );
     }
   }
-  else if ( item && item->depth() == 1 && col < widget->columns() )
+  else if ( item && item->depth() == 1 )
   {
-    m_menu->setItemEnabled( Password, true );
-    //m_menu->setItemEnabled( Preview, false );
-    //m_menu->setItemEnabled( Print, false );
-    m_menu->setItemEnabled( Mount, false );
+    m_collection->action( "askpass_action" )->setEnabled( true );
+    m_collection->action( "mount_action" )->setEnabled( false );
   }
   else
   {
-    //m_menu->setItemEnabled( Preview, false );
-    //m_menu->setItemEnabled( Print, false );
-    m_menu->setItemEnabled( Mount, false );
-    m_menu->setItemEnabled( Password, false );
+    m_collection->action( "mount_action" )->setEnabled( false );
+    m_collection->action( "askpass_action" )->setEnabled( false );
+  }
+}
+
+
+void KonqSidebar_Smb4K::slotRightButtonPressed( QListViewItem *item, const QPoint &point, int )
+{
+  if ( !item )
+  {
+    m_menu->popupMenu()->changeTitle( 0, SmallIcon( "network" ), i18n( "Network" ) );
+  }
+  else
+  {
+    m_menu->popupMenu()->changeTitle( 0, *(item->pixmap( 0 )), item->text( 0 ) );
+
+    if ( item->depth() == 0 )
+    {
+      m_collection->action( "rescan_action" )->setText( i18n( "Scan Wo&rkgroup" ) );
+    }
+    else
+    {
+      m_collection->action( "rescan_action" )->setText( i18n( "Scan Compute&r" ) );
+    }
   }
 
-  m_menu->exec( point, 0 );
+  slotSelectionChanged( item );
+
+  m_menu->popupMenu()->exec( point, 0 );
+
+  m_collection->action( "rescan_action" )->setText( i18n( "Scan Netwo&rk" ) );
 }
 
 
@@ -353,26 +368,23 @@ void KonqSidebar_Smb4K::slotMembers( const QValueList<Smb4KHostItem *> &list )
   {
     for ( QValueList<Smb4KHostItem *>::ConstIterator it = list.begin(); it != list.end(); ++it )
     {
-      QListViewItem *item = widget->findItem( (*it)->name(), Network, CaseSensitive|ExactMatch );
+      Smb4KBrowserWidgetItem *item = (Smb4KBrowserWidgetItem *)(widget->findItem( (*it)->name(), Network, CaseSensitive|ExactMatch ));
 
-      if ( item != 0 )
+      if ( item )
       {
-        if ( !((*it)->ip()).isEmpty() && QString::compare( item->text( IP ), (*it)->ip() ) != 0 )
-        {
-          item->setText( IP, (*it)->ip() );
-        }
-
-        if ( !((*it)->comment()).isEmpty() && QString::compare( item->text( Comment ), (*it)->comment() ) != 0 )
-        {
-          item->setText( Comment, (*it)->comment() );
-        }
+        item->update( *it );
 
         continue;
       }
       else
       {
-        Smb4KBrowserWidgetItem *host = new Smb4KBrowserWidgetItem( widget->findItem( (*it)->workgroup(), Network, ExactMatch|CaseSensitive ), *it );
-        host->setExpandable( true );
+        QListViewItem *workgroup = widget->findItem( (*it)->workgroup(), Network, ExactMatch|CaseSensitive );
+
+        if ( workgroup )
+        {
+          Smb4KBrowserWidgetItem *host = new Smb4KBrowserWidgetItem( workgroup, *it );
+          host->setExpandable( true );
+        }
       }
     }
   }
@@ -407,20 +419,20 @@ void KonqSidebar_Smb4K::slotShares( const QValueList<Smb4KShareItem *> &list )
           continue;
         }
 
-        QListViewItem *test = widget->findItem( (*it)->name(), Network, ExactMatch|CaseSensitive );
+        Smb4KBrowserWidgetItem *test = (Smb4KBrowserWidgetItem *)(widget->findItem( (*it)->name(), Network, ExactMatch|CaseSensitive ));
 
-        if ( test == 0 || test->parent() != host )
+        if ( !test || test->parent() != host )
         {
           Smb4KBrowserWidgetItem *share = new Smb4KBrowserWidgetItem( host, *it );
           share->setExpandable( false );
 
           continue;
         }
-        else if ( test != 0 && test->parent() == host )
+        else if ( test && test->parent() == host )
         {
-          if ( !((*it)->comment()).isEmpty() && QString::compare( test->text( Comment ), (*it)->comment() ) != 0 )
+          if ( QString::compare( test->text( Comment ), (*it)->comment() ) != 0 )
           {
-            test->setText( Comment, (*it)->comment() );
+            test->update( *it );
           }
 
           continue;
@@ -434,14 +446,7 @@ void KonqSidebar_Smb4K::slotMountShare()
 {
   if ( widget->currentItem() && widget->currentItem()->depth() == 2 )
   {
-    if ( QString::compare( widget->currentItem()->text( Network ).stripWhiteSpace(), "homes" ) == 0 )
-    {
-      QString name = specifyUser( widget->currentItem()->parent()->text( Network ) );
-      if ( !name.isEmpty() )
-        m_core->mounter()->mountShare( widget->currentItem()->parent()->parent()->text( Network ), widget->currentItem()->parent()->text( Network ), widget->currentItem()->parent()->text( IP ), name );
-    }
-    else
-      m_core->mounter()->mountShare( widget->currentItem()->parent()->parent()->text( Network ), widget->currentItem()->parent()->text( Network ), widget->currentItem()->parent()->text( IP ), widget->currentItem()->text( Network ) );
+    m_core->mounter()->mountShare( widget->currentItem()->parent()->parent()->text( Network ), widget->currentItem()->parent()->text( Network ), widget->currentItem()->parent()->text( IP ), widget->currentItem()->text( Network ) );
   }
 }
 
@@ -452,13 +457,11 @@ void KonqSidebar_Smb4K::slotUnmountShare()
  if ( i && i->depth() == 2 )
    {
     Smb4KShare *share = m_core->mounter()->findShareByName( QString( "//%1/%2" ).arg( i->parent()->text( Network ), i->text( Network ) ) );
-    tmpPath = share->getPath();
+    tmpPath = share->path();
     //kdDebug() << "KonqSidebar_Smb4K::slotUnmountShare - "<< tmpPath << endl;
     if ( QString::compare( tmpPath, currentKonquerorURL ) == 0 )
           emit openURLRequest( KURL( QDir::home().canonicalPath() ) );
-    m_core->mounter()->unmountShare( share->getPath(),
-                                           QString( "%1" ).arg( share->getUID() ),
-                                           QString( "%1" ).arg( share->getGID() ) );
+    m_core->mounter()->unmountShare( share );
    }
 }
 
@@ -484,16 +487,30 @@ void KonqSidebar_Smb4K::slotAskPass()
 
 void KonqSidebar_Smb4K::slotAddIPAddress( Smb4KHostItem *item )
 {
-  // First find the host and check wether it is the right one.
-  QListViewItem *hostItem = widget->findItem( item->name(), Network, ExactMatch|CaseSensitive );
-
-  if ( QString::compare( hostItem->parent()->text( Network ), item->workgroup() ) == 0 )
+  if ( item )
   {
-    hostItem->setText( IP, item->ip() );
+    Smb4KBrowserWidgetItem *workgroup = (Smb4KBrowserWidgetItem *)(widget->findItem( item->workgroup(), Network, ExactMatch|CaseSensitive ));
 
-    if ( widget->columnWidth( IP ) != 0 )
+    if ( workgroup && QString::compare( workgroup->text( Network ), item->workgroup() ) == 0 )
     {
-      widget->adjustColumn( IP );
+      Smb4KWorkgroupItem *wg = smb4k_core->scanner()->getWorkgroup( item->workgroup() );
+
+      if ( wg )
+      {
+        workgroup->update( wg );
+      }
+    }
+
+    Smb4KBrowserWidgetItem *hostItem = (Smb4KBrowserWidgetItem *)(widget->findItem( item->name(), Network, ExactMatch|CaseSensitive ));
+
+    if ( hostItem && hostItem->parent() && QString::compare( hostItem->parent()->text( Network ), item->workgroup() ) == 0 )
+    {
+      hostItem->update( item );
+
+      if ( widget->columnWidth( IP ) != 0 )
+      {
+        widget->adjustColumn( IP );
+      }
     }
   }
 }
@@ -546,85 +563,19 @@ void KonqSidebar_Smb4K::slotSmb4KOptionsDlg()
   if ( child( "ConfigDialog", "KDialogBase", true ) == 0 )
   {
     Smb4KOptionsDlg *configDlg = new Smb4KOptionsDlg( widget, "ConfigDialog", false );
-    connect( configDlg, SIGNAL( read() ), this, SLOT( slotReadOptions() ) );
+    connect( configDlg, SIGNAL( savedOptions() ), this, SLOT( slotReadOptions() ) );
     configDlg->show();
   }
 
 }
 
-const QString KonqSidebar_Smb4K::specifyUser( const QString &host )
+
+void KonqSidebar_Smb4K::slotRescan()
 {
-  KDialogBase *dialog = new KDialogBase( KDialogBase::Plain, i18n( "Specify User" ), KDialogBase::Ok|KDialogBase::Cancel, KDialogBase::Ok, widget, 0, false, true );
-
-  // Set up the ask pass dialog.
-  QFrame *frame = dialog->plainPage();
-  QGridLayout *layout = new QGridLayout( frame );
-  layout->setSpacing( 5 );
-
-  QLabel *pic = new QLabel( frame );
-  pic->setPixmap( DesktopIcon( "personal" ) );
-  pic->setMargin( 10 );
-
-  QLabel *text = new QLabel( i18n( "Please specify a user name." ), frame );
-
-  QLabel *user = new QLabel( i18n( "User:" ), frame );
-  KComboBox *username = new KComboBox( true, frame, 0 );
-  username->setDuplicatesEnabled( false );
-
-  QSpacerItem *spacer1 = new QSpacerItem( 10, 10, QSizePolicy::Expanding, QSizePolicy::Preferred );
-
-  layout->addWidget( pic, 0, 0, 0 );
-  layout->addMultiCellWidget( text, 0, 0, 1, 3, 0 );
-  layout->addWidget( user, 1, 0, 0 );
-  layout->addMultiCellWidget( username, 1, 1, 1, 4, 0 );
-  layout->addItem( spacer1, 0, 2 );
-
-  // Read the list of logins, that were already defined
-  // for this 'homes' share.
-  QStringList list;
-
-  if ( config()->hasGroup( "Homes Shares" ) )
-  {
-    config()->setGroup( "Homes Shares" );
-    if ( config()->hasKey( host ) )
-      list = config()->readListEntry( host, ',' );
-  }
-
-  if ( !list.isEmpty() )
-    username->insertStringList( list, -1 );
-
-  username->setCurrentText( QString::null );
-
-  // Do the last things before showing.
-  username->setFocus();
-  dialog->setFixedSize( dialog->sizeHint() );
-
-  if ( dialog->exec() == KDialogBase::Accepted )
-  {
-    // Write the new list of logins to the config file.
-    if ( list.contains( username->currentText() ) == 0 )
-      list.append( username->currentText() );
-
-    int index = 0;
-    while ( index < username->count() )
-    {
-      if ( list.contains( username->text( index ) ) == 0 )
-        list.append( username->text( index ) );
-
-      index++;
-    }
-
-    list.sort();
-
-    config()->setGroup( "Homes Shares" );
-    config()->writeEntry( host, list, ',' );
-
-    return username->currentText();
-  }
-  else
-    return QString::null;
+  smb4k_core->scanner()->rescan();
 }
 
+
 void KonqSidebar_Smb4K::slotReadOptions()
 {
   // Read the configuration.
