@@ -7,12 +7,20 @@
  ***************************************************************************/
 
 /***************************************************************************
- *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
+ *   This program is distributed in the hope that it will be useful, but   *
+ *   WITHOUT ANY WARRANTY; without even the implied warranty of            *
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
+ *   General Public License for more details.                              *
+ *                                                                         *
+ *   You should have received a copy of the GNU General Public License     *
+ *   along with this program; if not, write to the                         *
+ *   Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
+ *   MA  02111-1307 USA                                                    *
  ***************************************************************************/
 
 // Qt includes
@@ -27,20 +35,22 @@
 #include <kmessagebox.h>
 #include <kdialogbase.h>
 #include <kglobalsettings.h>
-#include <kactionclasses.h>
 #include <kpopupmenu.h>
 #include <kiconloader.h>
+#include <kdebug.h>
 
 // application specific includes
 #include "smb4kbrowserwidget.h"
 #include "smb4kpreviewdialog.h"
 #include "smb4kprintdialog.h"
+#include "smb4kmountdialog.h"
 #include "../core/smb4kshare.h"
 #include "../core/smb4kcore.h"
 #include "../core/smb4kpasswordhandler.h"
 #include "../core/smb4kglobal.h"
+#include "../core/smb4kdefs.h"
 
-using namespace Smb4K_Global;
+using namespace Smb4KGlobal;
 
 
 Smb4KBrowserWidget::Smb4KBrowserWidget( QWidget *parent, const char *name ) : KListView( parent, name )
@@ -48,6 +58,8 @@ Smb4KBrowserWidget::Smb4KBrowserWidget( QWidget *parent, const char *name ) : KL
   setFrameShape( KListView::PopupPanel );
   setFrameShadow( KListView::Sunken );
   setRootIsDecorated( true );
+  setAllColumnsShowFocus( false );
+  setMouseTracking( true );
 
   addColumn( i18n( "Network" ), -1 );
   addColumn( i18n( "Type" ), -1 );
@@ -56,6 +68,10 @@ Smb4KBrowserWidget::Smb4KBrowserWidget( QWidget *parent, const char *name ) : KL
 
   m_collection = new KActionCollection( this, "BrowserWidget_ActionCollection", KGlobal::instance() );
 
+  m_menu = new Smb4KBrowserActionMenu( Smb4KBrowserActionMenu::Full, m_collection, QString::null, QIconSet(), this, "BrowserWidget_ActionMenu" );
+
+  m_tooltip = NULL;
+
   initActions();
 
   // Widget specific connections.
@@ -67,78 +83,135 @@ Smb4KBrowserWidget::Smb4KBrowserWidget( QWidget *parent, const char *name ) : KL
            this, SLOT( slotItemExpandedCollapsed( QListViewItem * ) ) );
   connect( this, SIGNAL( selectionChanged( QListViewItem * ) ),
            this, SLOT( slotSelectionChanged( QListViewItem * ) ) );
-  connect( this, SIGNAL( clicked( QListViewItem * ) ),
-           this, SLOT( slotItemClicked( QListViewItem * ) ) );
+  connect( this, SIGNAL( pressed( QListViewItem * ) ),
+           this, SLOT( slotMouseButtonPressed( QListViewItem * ) ) );
   connect( this, SIGNAL( rightButtonPressed( QListViewItem *, const QPoint &, int ) ),
            this, SLOT( slotRightButtonPressed( QListViewItem *, const QPoint &, int ) ) );
 
   // External connections.
-  connect( smb4k_core->scanner(),  SIGNAL( workgroups( const QValueList<Smb4KWorkgroupItem *> & ) ),
-           this,                   SLOT( slotWorkgroups( const QValueList<Smb4KWorkgroupItem *> & ) ) );
-  connect( smb4k_core->scanner(),  SIGNAL( members( const QValueList<Smb4KHostItem *> & ) ),
-           this,                   SLOT( slotMembers( const QValueList<Smb4KHostItem *> & ) ) );
-  connect( smb4k_core->scanner(),  SIGNAL( shares( const QValueList<Smb4KShareItem *> & ) ),
-           this,                   SLOT( slotShares( const QValueList<Smb4KShareItem *> & ) ) );
-  connect( smb4k_core->ipFinder(), SIGNAL( ip_address( Smb4KHostItem * ) ),
-           this,                   SLOT( slotAddIPAddress( Smb4KHostItem * ) ) );
-  connect( smb4k_core->mounter(),  SIGNAL( updated() ),
-           this,                   SLOT( slotMarkShares() ) );
+  connect( smb4k_core->scanner(), SIGNAL( workgroups( const QValueList<Smb4KWorkgroupItem *> & ) ),
+           this,                  SLOT( slotWorkgroups( const QValueList<Smb4KWorkgroupItem *> & ) ) );
+  connect( smb4k_core->scanner(), SIGNAL( members( const QValueList<Smb4KHostItem *> & ) ),
+           this,                  SLOT( slotMembers( const QValueList<Smb4KHostItem *> & ) ) );
+  connect( smb4k_core->scanner(), SIGNAL( shares( const QValueList<Smb4KShareItem *> & ) ),
+           this,                  SLOT( slotShares( const QValueList<Smb4KShareItem *> & ) ) );
+  connect( smb4k_core->scanner(), SIGNAL( state( int ) ),
+           this,                  SLOT( slotReceiveScannerState( int ) ) );
+  connect( smb4k_core->scanner(), SIGNAL( ipAddress( Smb4KHostItem * ) ),
+           this,                  SLOT( slotAddIPAddress( Smb4KHostItem * ) ) );
+  connect( smb4k_core->scanner(), SIGNAL( info( Smb4KHostItem * ) ),
+           this,                  SLOT( slotAddInfo( Smb4KHostItem * ) ) );
+  connect( smb4k_core->mounter(), SIGNAL( updated() ),
+           this,                  SLOT( slotMarkShares() ) );
 }
 
 
 Smb4KBrowserWidget::~Smb4KBrowserWidget()
 {
+  delete m_menu;
+  delete m_tooltip;
+
+  uint index = 0;
+
+  while ( index < m_collection->count() )
+  {
+    delete m_collection->action( index++ );
+  }
+
+  m_collection->clear();
 }
 
 
 void Smb4KBrowserWidget::initActions()
 {
-  m_preview_action = new KAction( i18n( "Pre&view" ), "view_icon", CTRL+Key_V, this,          SLOT( slotPreview() ), m_collection, "network_preview" );
+  m_preview_action = m_menu->previewAction();
   m_preview_action->setGroup( "BrowserWidget" );
   m_preview_action->setEnabled( false );
+  connect( m_preview_action, SIGNAL( activated() ), this, SLOT( slotPreview() ) );
 
-  m_askpass_action = new KAction( i18n( "Au&thentication" ), "identity", CTRL+Key_T, this,    SLOT( slotAskPass() ), m_collection, "network_askpass" );
+  m_askpass_action = m_menu->askpassAction();
   m_askpass_action->setGroup( "BrowserWidget" );
   m_askpass_action->setEnabled( false );
+  connect( m_askpass_action, SIGNAL( activated() ), this, SLOT( slotAskPass() ) );
 
-  m_print_action = new KAction( i18n( "&Print File" ), "printer1", CTRL+Key_P, this,          SLOT( slotPrint() ), m_collection, "network_print" );
+  m_print_action = m_menu->printAction();
   m_print_action->setGroup( "BrowserWidget" );
   m_print_action->setEnabled( false );
+  connect( m_print_action, SIGNAL( activated() ), this, SLOT( slotPrint() ) );
 
-  m_mount_action = new KAction( i18n( "&Mount" ), "hdd_mount", CTRL+Key_M, this,              SLOT( slotMountShare() ), m_collection, "network_mount" );
+  m_mount_action = m_menu->mountAction();
   m_mount_action->setGroup( "BrowserWidget" );
   m_mount_action->setEnabled( false );
+  connect( m_mount_action, SIGNAL( activated() ), this, SLOT( slotMountShare() ) );
 
-  m_bookmark_action = new KAction( i18n( "&Add Bookmark" ), "bookmark_add", CTRL+Key_B, this, SLOT( slotAddBookmark() ), m_collection, "network_add_bookmark" );
+  m_bookmark_action = m_menu->bookmarkAction();
   m_bookmark_action->setGroup( "BrowserWidget" );
   m_bookmark_action->setEnabled( false );
+  connect( m_bookmark_action, SIGNAL( activated() ), this, SLOT( slotAddBookmark() ) );
 
-  m_rescan_action = new KAction( i18n( "&Rescan" ), "reload", CTRL+Key_R, this,               SLOT( slotRescan() ), m_collection, "network_rescan" );
+  m_rescan_action = m_menu->rescanAction();
   m_rescan_action->setGroup( "BrowserWidget" );
   m_rescan_action->setEnabled( true );
+  connect( m_rescan_action, SIGNAL( activated() ), this, SLOT( slotRescan() ) );
 
-  m_abort_action = new KAction( i18n( "&Abort" ) , "stop", CTRL+Key_A, this,                  SLOT( slotAbort() ), m_collection, "abort" );
+  m_abort_action = m_menu->abortAction();
   m_abort_action->setGroup( "BrowserWidget" );
   m_abort_action->setEnabled( false );
+  connect( m_abort_action, SIGNAL( activated() ), this, SLOT( slotAbort() ) );
 
-  m_search_action = new KAction( i18n( "&Search" ), "find", CTRL+Key_S, this,                 SLOT( slotSearch() ), m_collection, "network_search" );
+  m_search_action = m_menu->searchAction();
   m_search_action->setGroup( "BrowserWidget" );
   m_search_action->setEnabled( true );
+  connect( m_search_action, SIGNAL( activated() ), this, SLOT( slotSearch() ) );
+
+  m_manual_action = m_menu->manualMountAction();
+  m_manual_action->setGroup( "BrowserWidget" );
+  m_manual_action->setEnabled( true );
+  connect( m_manual_action, SIGNAL( activated() ), this, SLOT( slotManualMount() ) );
 }
 
 
 void Smb4KBrowserWidget::setOpen( QListViewItem *item, bool open )
 {
-  if ( open && item->depth() == 0 )
+  if ( open )
   {
-    Smb4KWorkgroupItem *i = smb4k_core->scanner()->getWorkgroup( item->text( Network ) );
-
-    smb4k_core->scanner()->getWorkgroupMembers( item->text( Network ), i->master(), i->ip() );
+    switch ( item->depth() )
+    {
+      case 0:
+      {
+        Smb4KWorkgroupItem *i = smb4k_core->scanner()->getWorkgroup( item->text( Network ) );
+        smb4k_core->scanner()->getWorkgroupMembers( item->text( Network ), i->master(), i->ip() );
+        break;
+      }
+      case 1:
+      {
+        smb4k_core->scanner()->getShares( item->parent()->text( Network ), item->text( Network ), item->text( IP ) );
+        break;
+      }
+      default:
+        break;
+    }
   }
-  else if ( open && item->depth() == 1 )
+  else
   {
-    smb4k_core->scanner()->getShares( item->parent()->text( Network ), item->text( Network ), item->text( IP ) );
-    smb4k_core->scanner()->getInfo( item->parent()->text( Network ), item->text( Network ), item->text( IP ) );
+    switch ( item->depth() )
+    {
+      case 1:
+      {
+        // Remove all children (shares).
+        QListViewItem *shareItem = item->firstChild();
+
+        while ( shareItem )
+        {
+          delete shareItem;
+          shareItem = item->firstChild();
+        }
+
+        break;
+      }
+      default:
+        break;
+    }
   }
 
   KListView::setOpen( item, open );
@@ -155,32 +228,40 @@ void Smb4KBrowserWidget::insertItem( Smb4KHostItem *item )
   // Now put the host into the list.
   QListViewItem *workgroupItem = findItem( item->workgroup(), Network, ExactMatch|CaseSensitive );
 
-  if ( workgroupItem != 0 )
+  if ( workgroupItem )
   {
     QListViewItem *hostItem = findItem( item->name(), Network, ExactMatch|CaseSensitive );
 
-    if ( hostItem == 0 )
+    if ( !hostItem )
     {
       Smb4KBrowserWidgetItem *newItem = new Smb4KBrowserWidgetItem( workgroupItem, item );
       newItem->setExpandable( true );
+
+      emit itemInserted( item );
     }
-    else if ( hostItem != 0 && QString::compare( hostItem->parent()->text( Network ), item->workgroup() ) != 0 )
+    else if ( hostItem && QString::compare( hostItem->parent()->text( Network ), item->workgroup() ) != 0 )
     {
       Smb4KBrowserWidgetItem *newItem = new Smb4KBrowserWidgetItem( workgroupItem, item );
       newItem->setExpandable( true );
+
+      emit itemInserted( item );
     }
     else
     {
-      KMessageBox::sorry( this, i18n( "This computer is already in the list." ), QString::null, KMessageBox::Notify );
+      // Do not need to do anything here. The user is notified by the search dialog if
+      // a host is already in the list.
     }
   }
   else
   {
-    Smb4KBrowserWidgetItem *newItem = new Smb4KBrowserWidgetItem( this, new Smb4KWorkgroupItem( item->workgroup(), item->name(), item->ip() ) );
-    newItem->setExpandable( true );
+    Smb4KBrowserWidgetItem *groupItem = new Smb4KBrowserWidgetItem( this, new Smb4KWorkgroupItem( item->workgroup(), item->name(), item->ip() ) );
+    groupItem->setExpandable( true );
 
-    Smb4KBrowserWidgetItem *hostItem = new Smb4KBrowserWidgetItem( newItem, item );
+    Smb4KBrowserWidgetItem *hostItem = new Smb4KBrowserWidgetItem( groupItem, item );
+    hostItem->hostItem()->setMaster( true );  // pseudo master
     hostItem->setExpandable( true );
+
+    emit itemInserted( item );
   }
 }
 
@@ -245,91 +326,77 @@ void Smb4KBrowserWidget::readOptions()
   m_ipc = config()->readBoolEntry( "Show IPC", false );
   m_admin = config()->readBoolEntry( "Show ADMIN", false );
   m_printer = config()->readBoolEntry( "Show Printer", true );
+  m_show_tooltip = config()->readBoolEntry( "Show Network Tooltip", true );
 }
 
 
-const QString Smb4KBrowserWidget::specifyUser( const QString &host )
+void Smb4KBrowserWidget::contentsMouseMoveEvent( QMouseEvent *e )
 {
-  KDialogBase *dialog = new KDialogBase( KDialogBase::Plain, i18n( "Specify User" ), KDialogBase::Ok|KDialogBase::Cancel, KDialogBase::Ok, this, 0, false, true );
+  m_pos = viewport()->mapFromGlobal( e->globalPos() );
 
-  // Set up the ask pass dialog.
-  QFrame *frame = dialog->plainPage();
-  QGridLayout *layout = new QGridLayout( frame );
-  layout->setSpacing( 5 );
+  Smb4KBrowserWidgetItem *item = (Smb4KBrowserWidgetItem *)itemAt( m_pos );
 
-  QLabel *pic = new QLabel( frame );
-  pic->setPixmap( DesktopIcon( "personal" ) );
-  pic->setMargin( 10 );
-
-  QLabel *text = new QLabel( i18n( "Please specify a user name." ), frame );
-
-  QLabel *user = new QLabel( i18n( "User:" ), frame );
-  KComboBox *username = new KComboBox( true, frame, 0 );
-  username->setDuplicatesEnabled( false );
-
-  QSpacerItem *spacer1 = new QSpacerItem( 10, 10, QSizePolicy::Expanding, QSizePolicy::Preferred );
-
-  layout->addWidget( pic, 0, 0, 0 );
-  layout->addMultiCellWidget( text, 0, 0, 1, 3, 0 );
-  layout->addWidget( user, 1, 0, 0 );
-  layout->addMultiCellWidget( username, 1, 1, 1, 4, 0 );
-  layout->addItem( spacer1, 0, 2 );
-
-  // Read the list of logins, that were already defined
-  // for this 'homes' share.
-  QStringList list;
-
-  if ( config()->hasGroup( "Homes Shares" ) )
+  if ( item && m_show_tooltip )
   {
-    config()->setGroup( "Homes Shares" );
+    // Check if we are on the root decoration.
+    bool on_root = true;
 
-    if ( config()->hasKey( host ) )
+    if ( m_pos.x() > header()->sectionPos( header()->mapToIndex( 0 ) ) +
+         treeStepSize() * ( item->depth() + ( rootIsDecorated() ? 1 : 0 ) ) + itemMargin() ||
+         m_pos.x() < header()->sectionPos( header()->mapToIndex( 0 ) ) )
     {
-      list = config()->readListEntry( host, ',' );
-    }
-  }
-
-  if ( !list.isEmpty() )
-  {
-    username->insertStringList( list, -1 );
-  }
-
-  username->setCurrentText( QString::null );
-
-  // Do the last things before showing.
-  username->setFocus();
-  dialog->setFixedSize( dialog->sizeHint() );
-
-  if ( dialog->exec() == KDialogBase::Accepted )
-  {
-    // Write the new list of logins to the config file.
-    if ( list.contains( username->currentText() ) == 0 )
-    {
-      list.append( username->currentText() );
+      on_root = false;
     }
 
-    int index = 0;
-    while ( index < username->count() )
+    // Show the tool tip if there is an item, the list view has the mouse,
+    // and the user wants to see it. Additionally, make sure that it is only shown,
+    // if we are on the executable area (here: first column) and not on the root
+    // decoration.
+    if ( hasMouse() && isExecuteArea( m_pos ) /* first column */ && !on_root )
     {
-      if ( list.contains( username->text( index ) ) == 0 )
+      if ( !m_tooltip )
       {
-        list.append( username->text( index ) );
+        m_tooltip = new Smb4KNetworkItemTooltip( item, e->globalPos() );
+
+        QTimer::singleShot( 2000, this, SLOT( slotShowTooltip() ) );
       }
+      else if ( m_tooltip )
+      {
+        if ( m_tooltip->item() != item )
+        {
+          // If we get here, the user moved away from the previous
+          // item. So, we will kill the process, that should retrieve
+          // the additional info for it.
+          if ( smb4k_core->scannerIsRunning() &&
+               smb4k_core->scannerState() == SCANNER_RETRIEVING_INFO )
+          {
+            smb4k_core->scanner()->abort();
+          }
 
-      index++;
+          delete m_tooltip;
+          m_tooltip = NULL;
+        }
+      }
+    }
+    else
+    {
+      if ( m_tooltip )
+      {
+        delete m_tooltip;
+        m_tooltip = NULL;
+      }
     }
-
-    list.sort();
-
-    config()->setGroup( "Homes Shares" );
-    config()->writeEntry( host, list, ',' );
-
-    return username->currentText();
   }
   else
   {
-    return QString::null;
+    if ( m_tooltip )
+    {
+      delete m_tooltip;
+      m_tooltip = NULL;
+    }
   }
+
+  KListView::contentsMouseMoveEvent( e );
 }
 
 
@@ -337,9 +404,43 @@ const QString Smb4KBrowserWidget::specifyUser( const QString &host )
 // SLOT IMPLEMENTATIONS
 /////////////////////////////////////////////////////////////////////////////
 
+void Smb4KBrowserWidget::slotShowTooltip()
+{
+  if ( m_tooltip )
+  {
+    if ( m_tooltip->item()->depth() == 1 &&
+         m_tooltip->item() == (Smb4KBrowserWidgetItem *)itemAt( m_pos ) )
+    {
+      // We do not need to do additional checks here, because
+      // the scanner will only start a process, if this item wasn't
+      // checked for additional information already.
+      smb4k_core->scanner()->getInfo( m_tooltip->item()->hostItem()->workgroup(),
+                                      m_tooltip->item()->hostItem()->name(),
+                                      m_tooltip->item()->hostItem()->ip() );
+    }
+
+    if ( hasMouse() && isExecuteArea( m_pos ) )
+    {
+      m_tooltip->update();
+      m_tooltip->showTip();
+    }
+    else
+    {
+      delete m_tooltip;
+      m_tooltip = NULL;
+    }
+  }
+}
+
 
 void Smb4KBrowserWidget::slotItemExecuted( QListViewItem *item )
 {
+  if ( m_tooltip )
+  {
+    delete m_tooltip;
+    m_tooltip = NULL;
+  }
+
   if ( item )
   {
     if ( item->depth() == 2 )
@@ -372,6 +473,7 @@ void Smb4KBrowserWidget::slotSelectionChanged( QListViewItem *item )
   if ( item && item->depth() == 2 )
   {
     m_preview_action->setEnabled( true );
+    m_askpass_action->setEnabled( true );
 
     if ( item->text( Type ).contains( "Printer" ) == 0 )
     {
@@ -404,39 +506,43 @@ void Smb4KBrowserWidget::slotSelectionChanged( QListViewItem *item )
     m_mount_action->setEnabled( false );
     m_askpass_action->setEnabled( false );
   }
+}
 
-  if ( item )
+
+void Smb4KBrowserWidget::slotRightButtonPressed( QListViewItem *item, const QPoint &point, int )
+{
+  if ( !item )
   {
-    emit highlightedItem( item );
+    m_menu->popupMenu()->changeTitle( 0, SmallIcon( "network" ), i18n( "Network" ) );
   }
+  else
+  {
+    m_menu->popupMenu()->changeTitle( 0, *(item->pixmap( 0 )), item->text( 0 ) );
+
+    if ( item->depth() == 0 )
+    {
+      m_rescan_action->setText( i18n( "Scan Wo&rkgroup" ) );
+    }
+    else
+    {
+      m_rescan_action->setText( i18n( "Scan Compute&r" ) );
+    }
+  }
+
+  m_menu->popupMenu()->exec( point, 0 );
+
+  m_rescan_action->setText( i18n( "Scan Netwo&rk" ) );
 }
 
 
-void Smb4KBrowserWidget::slotRightButtonPressed( QListViewItem *, const QPoint &point, int )
+void Smb4KBrowserWidget::slotMouseButtonPressed( QListViewItem *item )
 {
-  KActionMenu *menu = new KActionMenu( this, "BrowserPopupMenu" );
+    if ( m_tooltip )
+  {
+    delete m_tooltip;
+    m_tooltip = NULL;
+  }
 
-  menu->popupMenu()->insertTitle( SmallIcon( "network" ), i18n( "Network" ), 0, 0 );
-  menu->insert( m_rescan_action );
-  menu->insert( m_abort_action );
-  menu->popupMenu()->insertSeparator( -1 );
-  menu->insert( m_search_action );
-  menu->popupMenu()->insertSeparator( -1 );
-  menu->insert( m_bookmark_action );
-  menu->insert( m_askpass_action );
-  menu->insert( m_preview_action );
-  menu->insert( m_print_action );
-  menu->insert( m_mount_action );
-
-  menu->popupMenu()->exec( point, 0 );
-}
-
-
-void Smb4KBrowserWidget::slotItemClicked( QListViewItem *item )
-{
-  // We only need this for the case, that the user clicks somewhere, where
-  // NO item is. Smb4KNetworkTab needs it to clear the tab and the main window
-  // to disable some actions in the tool bar.
   if ( !item && !selectedItem() )
   {
     m_bookmark_action->setEnabled( false );
@@ -444,8 +550,6 @@ void Smb4KBrowserWidget::slotItemClicked( QListViewItem *item )
     m_print_action->setEnabled( false );
     m_mount_action->setEnabled( false );
     m_askpass_action->setEnabled( false );
-
-    emit highlightedItem( item );
   }
 }
 
@@ -455,8 +559,6 @@ void Smb4KBrowserWidget::slotWorkgroups( const QValueList<Smb4KWorkgroupItem *>
   // Clear the list view:
   clear();
 
-  emit cleared();
-
   // Adjust the columns.
   for ( int index = 0; index < columns(); index++ )
   {
@@ -471,7 +573,7 @@ void Smb4KBrowserWidget::slotWorkgroups( const QValueList<Smb4KWorkgroupItem *>
   {
     for ( QValueList<Smb4KWorkgroupItem *>::ConstIterator it = list.begin(); it != list.end(); ++it )
     {
-      if ( this->findItem( (*it)->workgroup(), Network, CaseSensitive|ExactMatch ) == 0 )
+      if ( findItem( (*it)->workgroup(), Network, CaseSensitive|ExactMatch ) == 0 )
       {
         Smb4KBrowserWidgetItem *workgroup = new Smb4KBrowserWidgetItem( this, *it );
         workgroup->setExpandable( true );
@@ -492,26 +594,23 @@ void Smb4KBrowserWidget::slotMembers( const QValueList<Smb4KHostItem *> &list )
   {
     for ( QValueList<Smb4KHostItem *>::ConstIterator it = list.begin(); it != list.end(); ++it )
     {
-      QListViewItem *item = this->findItem( (*it)->name(), Network, CaseSensitive|ExactMatch );
+      Smb4KBrowserWidgetItem *item = (Smb4KBrowserWidgetItem *)findItem( (*it)->name(), Network, CaseSensitive|ExactMatch );
 
-      if ( item != 0 )
+      if ( item )
       {
-        if ( !((*it)->ip()).isEmpty() && QString::compare( item->text( IP ), (*it)->ip() ) != 0 )
-        {
-          item->setText( IP, (*it)->ip() );
-        }
-
-        if ( !((*it)->comment()).isEmpty() && QString::compare( item->text( Comment ), (*it)->comment() ) != 0 )
-        {
-          item->setText( Comment, (*it)->comment() );
-        }
+        item->update( *it );
 
         continue;
       }
       else
       {
-        Smb4KBrowserWidgetItem *host = new Smb4KBrowserWidgetItem( this->findItem( (*it)->workgroup(), Network, ExactMatch|CaseSensitive ), *it );
-        host->setExpandable( true );
+        QListViewItem *workgroup = findItem( (*it)->workgroup(), Network, ExactMatch|CaseSensitive );
+
+        if ( workgroup )
+        {
+          Smb4KBrowserWidgetItem *host = new Smb4KBrowserWidgetItem( workgroup, *it );
+          host->setExpandable( true );
+        }
       }
     }
   }
@@ -524,42 +623,46 @@ void Smb4KBrowserWidget::slotShares( const QValueList<Smb4KShareItem *> &list )
   {
     for ( QValueList<Smb4KShareItem *>::ConstIterator it = list.begin(); it != list.end(); ++it )
     {
-      QListViewItem *host = this->findItem( (*it)->host(), Network, ExactMatch|CaseSensitive );
+      QListViewItem *host = findItem( (*it)->host(), Network, ExactMatch|CaseSensitive );
 
       if ( host != 0 )
       {
-        if ( !m_hidden && (*it)->name().endsWith( "$" ) )
+        if ( !m_hidden && (*it)->name().stripWhiteSpace().endsWith( "$" ) )
         {
           continue;
         }
         else
         {
           if ( !m_ipc && (*it)->name().contains( "IPC" ) != 0 )
+          {
             continue;
+          }
 
           if ( !m_admin && (*it)->name().contains( "ADMIN" ) != 0 )
+          {
             continue;
+          }
         }
 
-        if ( !m_printer && QString::compare( (*it)->name(), "Printer" ) == 0 )
+        if ( !m_printer && QString::compare( (*it)->plainType(), "Printer" ) == 0 )
         {
           continue;
         }
 
-        QListViewItem *test = findItem( (*it)->name(), Network, ExactMatch|CaseSensitive );
+        Smb4KBrowserWidgetItem *test = (Smb4KBrowserWidgetItem *)findItem( (*it)->name(), Network, ExactMatch|CaseSensitive );
 
-        if ( test == 0 || test->parent() != host )
+        if ( !test || test->parent() != host )
         {
           Smb4KBrowserWidgetItem *share = new Smb4KBrowserWidgetItem( host, *it );
           share->setExpandable( false );
 
           continue;
         }
-        else if ( test != 0 && test->parent() == host )
+        else if ( test && test->parent() == host )
         {
-          if ( !((*it)->comment()).isEmpty() && QString::compare( test->text( Comment ), (*it)->comment() ) != 0 )
+          if ( QString::compare( test->text( Comment ), (*it)->comment() ) != 0 )
           {
-            test->setText( Comment, (*it)->comment() );
+            test->update( *it );
           }
 
           continue;
@@ -574,19 +677,7 @@ void Smb4KBrowserWidget::slotMountShare()
 {
   if ( currentItem() && currentItem()->depth() == 2 )
   {
-    if ( QString::compare( currentItem()->text( Network ).stripWhiteSpace(), "homes" ) == 0 )
-    {
-      QString name = specifyUser( currentItem()->parent()->text( Network ) );
-
-      if ( !name.isEmpty() )
-      {
-        smb4k_core->mounter()->mountShare( currentItem()->parent()->parent()->text( Network ), currentItem()->parent()->text( Network ), currentItem()->parent()->text( IP ), name );
-      }
-    }
-    else
-    {
-      smb4k_core->mounter()->mountShare( currentItem()->parent()->parent()->text( Network ), currentItem()->parent()->text( Network ), currentItem()->parent()->text( IP ), currentItem()->text( Network ) );
-    }
+    smb4k_core->mounter()->mountShare( currentItem()->parent()->parent()->text( Network ), currentItem()->parent()->text( Network ), currentItem()->parent()->text( IP ), currentItem()->text( Network ) );
   }
 }
 
@@ -617,19 +708,13 @@ void Smb4KBrowserWidget::slotPreview()
   QString workgroup = currentItem()->parent()->parent()->text( Network );
   QString host = currentItem()->parent()->text( Network );
   QString ip = currentItem()->parent()->text( IP );
-  QString share;
+  QString share = currentItem()->text( Network );
 
-  if ( QString::compare( currentItem()->text( Network ).stripWhiteSpace(), "homes" ) == 0 )
-  {
-    share = specifyUser( currentItem()->parent()->text( Network ) );
-  }
-  else
-  {
-    share = currentItem()->text( Network );
-  }
-
-  Smb4KPreviewDialog *preview = new Smb4KPreviewDialog( workgroup, host, ip, share, this, 0, false );
+  Smb4KPreviewDialog *preview = new Smb4KPreviewDialog( workgroup, host, ip, share, 0, 0, false );
   preview->show();
+
+  // FIXME: Do we produce a memory leak here? We have Qt::WDestructiveClose set in
+  // Smb4KPreviewDialog.
 }
 
 
@@ -637,42 +722,97 @@ void Smb4KBrowserWidget::slotAddBookmark()
 {
   if ( currentItem() && currentItem()->depth() == 2 )
   {
-    if ( QString::compare( currentItem()->text( Network ).stripWhiteSpace(), "homes" ) != 0 )
-    {
-      Smb4KBookmark *bookmark = new Smb4KBookmark( QString( "//%1/%2" ).arg( currentItem()->parent()->text( Network ), currentItem()->text( Network ) ), currentItem()->text( Type ) );
-      smb4k_core->bookmarkHandler()->addBookmark( bookmark );
-    }
-    else
-    {
-      QString share = specifyUser( currentItem()->parent()->text( Network ) );
-      Smb4KBookmark *bookmark = new Smb4KBookmark( QString( "//%1/%2" ).arg( currentItem()->parent()->text( Network ), share ), currentItem()->text( Type ) );
-      smb4k_core->bookmarkHandler()->addBookmark( bookmark );
-    }
+    QString host = currentItem()->parent()->text( Network );
+    QString share = currentItem()->text( Network );
+    QString workgroup = currentItem()->parent()->parent()->text( Network );
+    QString ip = currentItem()->parent()->text( IP );
+    QString type = currentItem()->text( Type );
+
+    Smb4KBookmark *bookmark = new Smb4KBookmark( host, share, workgroup, ip, type );
+    smb4k_core->bookmarkHandler()->addBookmark( bookmark );
   }
 }
 
 
 void Smb4KBrowserWidget::slotRescan()
 {
-  smb4k_core->scanner()->rescan();
+  bool viewport_has_mouse;
+
+  if ( m_pos.x() <= 0 || viewport()->width() <= m_pos.x() ||
+       m_pos.y() <= 0 || viewport()->height() <= m_pos.y() )
+  {
+    viewport_has_mouse = false;
+  }
+  else
+  {
+    viewport_has_mouse = true;
+  }
+
+  if ( currentItem() && selectedItem() && viewport_has_mouse )
+  {
+    if ( currentItem()->depth() == 0 )
+    {
+      if ( !currentItem()->isOpen() )
+      {
+        setOpen( currentItem(), true );
+      }
+      else
+      {
+        Smb4KWorkgroupItem *item = ((Smb4KBrowserWidgetItem *)currentItem())->workgroupItem();
+        smb4k_core->scanner()->getWorkgroupMembers( item->workgroup(), item->master(), item->ip() );
+      }
+    }
+    else if ( currentItem()->depth() == 1 )
+    {
+      if ( !currentItem()->isOpen() )
+      {
+        setOpen( currentItem(), true );
+      }
+      else
+      {
+        Smb4KHostItem *item = ((Smb4KBrowserWidgetItem *)currentItem())->hostItem();
+        smb4k_core->scanner()->getShares( item->workgroup(), item->name(), item->ip() );
+      }
+    }
+    else
+    {
+      Smb4KHostItem *item = ((Smb4KBrowserWidgetItem *)currentItem()->parent())->hostItem();
+      smb4k_core->scanner()->getShares( item->workgroup(), item->name(), item->ip() );
+    }
+  }
+  else
+  {
+    smb4k_core->scanner()->rescan();
+  }
 }
 
 
 void Smb4KBrowserWidget::slotAddIPAddress( Smb4KHostItem *item )
 {
-  // First find the host and check wether it is the right one.
-  QListViewItem *hostItem = findItem( item->name(), Network, ExactMatch|CaseSensitive );
-
-  // FIXME: This might be a potential problem, if there are several hosts
-  // with the same name in a domain/LAN.
-  // *** Question: Is this case at all possible? ***
-  if ( hostItem && hostItem->parent() && QString::compare( hostItem->parent()->text( Network ), item->workgroup() ) == 0 )
+  if ( item )
   {
-    hostItem->setText( IP, item->ip() );
+    Smb4KBrowserWidgetItem *workgroup = (Smb4KBrowserWidgetItem *)findItem( item->workgroup(), Network, ExactMatch|CaseSensitive );
 
-    if ( this->columnWidth( IP ) != 0 )
+    if ( workgroup && QString::compare( workgroup->text( Network ), item->workgroup() ) == 0 )
     {
-      adjustColumn( IP );
+      Smb4KWorkgroupItem *wg = smb4k_core->scanner()->getWorkgroup( item->workgroup() );
+
+      if ( wg )
+      {
+        workgroup->update( wg );
+      }
+    }
+
+    Smb4KBrowserWidgetItem *hostItem = (Smb4KBrowserWidgetItem *)findItem( item->name(), Network, ExactMatch|CaseSensitive );
+
+    if ( hostItem && hostItem->parent() && QString::compare( hostItem->parent()->text( Network ), item->workgroup() ) == 0 )
+    {
+      hostItem->update( item );
+
+      if ( columnWidth( IP ) != 0 )
+      {
+        adjustColumn( IP );
+      }
     }
   }
 }
@@ -742,7 +882,55 @@ void Smb4KBrowserWidget::slotInsertItem( Smb4KHostItem *item )
 
 void Smb4KBrowserWidget::slotAbort()
 {
-  smb4k_core->abort();
+  smb4k_core->scanner()->abort();
+}
+
+
+void Smb4KBrowserWidget::slotManualMount()
+{
+  Smb4KMountDialog *dlg = new Smb4KMountDialog( 0, "ManualMountDialog" );
+
+  dlg->show();
+
+  // The dialog will be closed distructively.
+}
+
+
+void Smb4KBrowserWidget::slotReceiveScannerState( int state )
+{
+  if ( state != SCANNER_STOP )
+  {
+    if ( state != SCANNER_SEARCHING  )
+    {
+      m_rescan_action->setEnabled( false );
+    }
+
+    m_abort_action->setEnabled( true );
+  }
+  else
+  {
+    m_rescan_action->setEnabled( true );
+    m_abort_action->setEnabled( false );
+  }
+}
+
+
+void Smb4KBrowserWidget::slotAddInfo( Smb4KHostItem *item )
+{
+  if ( item )
+  {
+    Smb4KBrowserWidgetItem *host = (Smb4KBrowserWidgetItem *)findItem( item->name(), Network, ExactMatch|CaseSensitive );
+
+    if ( host && QString::compare( host->parent()->text( Network ), item->workgroup() ) == 0 )
+    {
+      host->update( item );
+    }
+
+    if ( m_tooltip && m_tooltip->item() == host )
+    {
+      m_tooltip->update();
+    }
+  }
 }
 
 #include "smb4kbrowserwidget.moc"
