@@ -7,12 +7,20 @@
  ***************************************************************************/
 
 /***************************************************************************
- *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
+ *   This program is distributed in the hope that it will be useful, but   *
+ *   WITHOUT ANY WARRANTY; without even the implied warranty of            *
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
+ *   General Public License for more details.                              *
+ *                                                                         *
+ *   You should have received a copy of the GNU General Public License     *
+ *   along with this program; if not, write to the                         *
+ *   Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
+ *   MA  02111-1307 USA                                                    *
  ***************************************************************************/
 
 // Qt includes
@@ -28,8 +36,6 @@
 // application specific includes
 #include "smb4kmountdialog.h"
 
-#include "smb4kmountdialog.moc"
-
 #include "../core/smb4kmounter.h"
 #include "../core/smb4kbookmarkhandler.h"
 #include "../core/smb4kbookmark.h"
@@ -54,24 +60,32 @@ void Smb4KMountDialog::setupView()
 {
   QFrame *frame = plainPage();
   QGridLayout *layout = new QGridLayout( frame );
+  layout->setSpacing( 10 );
+  layout->setMargin( 0 );
 
-  QGroupBox *input_box = new QGroupBox( 2, Qt::Horizontal, i18n( "Input" ), frame, "InputBox" );
-  new QLabel( i18n( "Share:" ), input_box );
-  m_share_input = new KLineEdit( input_box, "ShareInputLine" );
+  QLabel *shareLabel = new QLabel( i18n( "Share:" ), frame );
+  m_share_input = new KLineEdit( frame, "ShareInputLine" );
   m_share_input->setMinimumWidth( 200 );
   m_share_input->setFocus();
-  new QLabel( i18n( "IP Address:" ), input_box );
-  m_ip_input = new KLineEdit( input_box, "IPInputLine" );
+
+  QLabel *addressLabel = new QLabel( i18n( "IP Address:" ), frame );
+  m_ip_input = new KLineEdit( frame, "IPInputLine" );
   m_ip_input->setMinimumWidth( 200 );
-  new QLabel( i18n( "Workgroup:" ), input_box );
-  m_workgroup_input = new KLineEdit( input_box, "WorkgroupInputLine" );
+
+  QLabel *workgroupLabel = new QLabel( i18n( "Workgroup:" ), frame );
+  m_workgroup_input = new KLineEdit( frame, "WorkgroupInputLine" );
   m_workgroup_input->setMinimumWidth( 200 );
 
-  QGroupBox *bookmark_box = new QGroupBox( 1, Qt::Horizontal, i18n( "Bookmark" ), frame, "BookmarkBox" );
-  m_bookmark = new QCheckBox( i18n( "Add this share to the bookmarks" ), bookmark_box, "BookmarkButton" );
+  m_bookmark = new QCheckBox( i18n( "Add this share to the bookmarks" ), frame, "BookmarkButton" );
+
+  layout->addWidget( shareLabel, 0, 0, 0 );
+  layout->addWidget( m_share_input, 0, 1, 0 );
+  layout->addWidget( addressLabel, 1, 0, 0 );
+  layout->addWidget( m_ip_input, 1, 1, 0 );
+  layout->addWidget( workgroupLabel, 2, 0, 0 );
+  layout->addWidget( m_workgroup_input, 2, 1, 0 );
+  layout->addMultiCellWidget( m_bookmark, 3, 3, 0, 1, 0 );
 
-  layout->addWidget( input_box, 0, 0, 0 );
-  layout->addWidget( bookmark_box, 1, 0, 0 );
   connect( m_share_input, SIGNAL( textChanged ( const QString & ) ) , this, SLOT( slotChangeInputValue( const QString & ) ) );
   slotChangeInputValue( m_share_input->text() );
 }
@@ -97,7 +111,7 @@ void Smb4KMountDialog::slotOk()
          && m_share_input->text().contains( '@' ) == 0 )
 #endif
     {
-      QString host = m_share_input->text().stripWhiteSpace().section( "/", 2, 2 ).section( "/", 0, 0 );
+      QString host = m_share_input->text().stripWhiteSpace().section( "/", 2, 2 );
       QString share = m_share_input->text().stripWhiteSpace().section( "/", 3, 3 );
       QString ip = m_ip_input->text().stripWhiteSpace();
       QString workgroup = m_workgroup_input->text().stripWhiteSpace();
@@ -106,7 +120,7 @@ void Smb4KMountDialog::slotOk()
 
       if ( m_bookmark->isChecked() )
       {
-        smb4k_core->bookmarkHandler()->addBookmark( new Smb4KBookmark( m_share_input->text().stripWhiteSpace(), QString::null ) );
+        smb4k_core->bookmarkHandler()->addBookmark( new Smb4KBookmark( host, share, workgroup, ip, QString::null ) );
       }
 
       accept();
@@ -117,3 +131,6 @@ void Smb4KMountDialog::slotOk()
     }
   }
 }
+
+#include "smb4kmountdialog.moc"
+
