@@ -7,19 +7,26 @@
  ***************************************************************************/
 
 /***************************************************************************
- *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
+ *   This program is distributed in the hope that it will be useful, but   *
+ *   WITHOUT ANY WARRANTY; without even the implied warranty of            *
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
+ *   General Public License for more details.                              *
+ *                                                                         *
+ *   You should have received a copy of the GNU General Public License     *
+ *   along with this program; if not, write to the                         *
+ *   Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
+ *   MA  02111-1307 USA                                                    *
  ***************************************************************************/
 
 // Qt includes
 #include <qlayout.h>
 #include <qtooltip.h>
 #include <qiconset.h>
-#include <qregexp.h>
 
 // KDE includes
 #include <klocale.h>
@@ -30,19 +37,15 @@
 #include "smb4kpreviewdialog.h"
 #include "../smb4k.h"
 #include "../core/smb4kcore.h"
+#include "../core/smb4kglobal.h"
+
+using namespace Smb4KGlobal;
 
 
 Smb4KPreviewDialog::Smb4KPreviewDialog( const QString &workgroup, const QString &host, const QString &ip,
 const QString &share, QWidget *parent, const char *name, bool modal )
-: KDialogBase( Plain, i18n( "Preview" ), Close, Close, parent, name, modal, true )
+: KDialogBase( Plain, i18n( "Preview" ), Close, Close, parent, name, modal, true ), m_workgroup( workgroup ), m_host( host ), m_share( share ), m_ip( ip )
 {
-  m_workgroup = workgroup;
-  m_host = host;
-  m_share = share;
-  m_ip = ip;
-
-  m_address = QString( "//%1/%2/" ).arg( m_host ).arg( m_share );
-
   setupView();
 
   readOptions();
@@ -50,14 +53,26 @@ const QString &share, QWidget *parent, const char *name, bool modal )
   setMinimumSize( sizeHint() );
   setWFlags( Qt::WDestructiveClose );
 
-  connect( this,                  SIGNAL( finished() ), this,                   SLOT( slotFinished() ) );
-  connect( m_view,                SIGNAL( executed( QIconViewItem * ) ), this,  SLOT( slotItemExecuted( QIconViewItem * ) ) );
-  connect( m_toolbar,             SIGNAL( clicked( int ) ), this,               SLOT( slotButtonClicked( int ) ) );
-  connect( m_combo,               SIGNAL( activated( const QString & ) ), this, SLOT( slotItemActivated( const QString & ) ) );
+  connect( this,      SIGNAL( finished() ),
+           this,      SLOT( slotFinished() ) );
+  connect( m_view,    SIGNAL( executed( QIconViewItem * ) ),
+           this,      SLOT( slotItemExecuted( QIconViewItem * ) ) );
+  connect( m_toolbar, SIGNAL( clicked( int ) ),
+           this,      SLOT( slotButtonClicked( int ) ) );
+  connect( m_combo,   SIGNAL( activated( const QString & ) ),
+           this,      SLOT( slotItemActivated( const QString & ) ) );
 
-  connect( smb4k_core->scanner(), SIGNAL( previewResult( const QValueList<Smb4KPreviewItem *> & ) ), this, SLOT( slotReceivedData( const QValueList<Smb4KPreviewItem *> & ) ) );
+  connect( smb4k_core->scanner(), SIGNAL( previewResult( const QValueList<Smb4KPreviewItem *> & ) ),
+           this,                  SLOT( slotReceivedData( const QValueList<Smb4KPreviewItem *> & ) ) );
 
-  smb4k_core->scanner()->getPreview( workgroup, host, ip, share, QString::null );
+  m_share = smb4k_core->scanner()->getPreview( workgroup, host, ip, share, QString::null );
+
+  m_address = QString( "//%1/%2/" ).arg( m_host ).arg( m_share );
+
+  if ( m_share.isEmpty() )
+  {
+    close();
+  }
 }
 
 
@@ -97,14 +112,14 @@ void Smb4KPreviewDialog::setupView()
 
 void Smb4KPreviewDialog::saveOptions()
 {
-  config()->setGroup( "General Options" );
+  config()->setGroup( "Dialogs" );
   config()->writeEntry( "Preview Dialog Geometry", size() );
 }
 
 
 void Smb4KPreviewDialog::readOptions()
 {
-  config()->setGroup( "General Options" );
+  config()->setGroup( "Dialogs" );
   setInitialSize( config()->readSizeEntry( "Preview Dialog Geometry", 0L ) );
 }
 
@@ -180,12 +195,12 @@ void Smb4KPreviewDialog::slotReceivedData( const QValueList<Smb4KPreviewItem *>
     {
       if ( (*it)->isFile() )
       {
-        QIconViewItem *view_item = new QIconViewItem( m_view, (*it)->name(), SmallIcon( "misc" ) );
+        QIconViewItem *view_item = new QIconViewItem( m_view, (*it)->name(), SmallIcon( "file" ) );
         view_item->setKey( QString( "1%1" ).arg( (*it)->name() ) );
       }
       else
       {
-        if ( QString::compare( (*it)->name(), "." ) != 0 && QString::compare( (*it)->name(), ".." ) !=0 )
+        if ( QString::compare( (*it)->name(), "." ) != 0 && QString::compare( (*it)->name(), ".." ) != 0 )
         {
           QIconViewItem *view_item = new QIconViewItem( m_view, (*it)->name(), SmallIcon( "folder" ) );
           view_item->setKey( QString( "0%1" ).arg( (*it)->name() ) );
