@@ -7,12 +7,20 @@
  ***************************************************************************/
 
 /***************************************************************************
- *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
+ *   This program is distributed in the hope that it will be useful, but   *
+ *   WITHOUT ANY WARRANTY; without even the implied warranty of            *
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
+ *   General Public License for more details.                              *
+ *                                                                         *
+ *   You should have received a copy of the GNU General Public License     *
+ *   along with this program; if not, write to the                         *
+ *   Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
+ *   MA  02111-1307 USA                                                    *
  ***************************************************************************/
 
 // Qt includes
@@ -27,6 +35,7 @@
 // application specific includes
 #include "smb4ksuperuseroptions.h"
 #include "../smb4k.h"
+#include "../core/smb4kcore.h"
 
 
 Smb4KSuperUserOptions::Smb4KSuperUserOptions( QWidget *parent, const char *name )
@@ -34,27 +43,27 @@ Smb4KSuperUserOptions::Smb4KSuperUserOptions( QWidget *parent, const char *name
 {
   QGridLayout *grid = new QGridLayout( this );
   grid->setSpacing( 10 );
-  
-  QButtonGroup *suid_progs = new QButtonGroup( 1, QGroupBox::Horizontal, i18n( "Programs" ), this, "SUIDPrograms" );
+
+  QButtonGroup *suid_progs = new QButtonGroup( 1, QButtonGroup::Horizontal, i18n( "Programs" ), this, "SUIDPrograms" );
   suid_progs->setInsideMargin( 10 );
   new QLabel( i18n( "Use the following program to gain super user privileges:" ), suid_progs );
-  m_super = new QRadioButton( i18n( "super (recommended)" ), suid_progs, "SuperButton" );
+  m_super = new QRadioButton( "super", suid_progs, "SuperButton" );
   m_super->setChecked( true );
-  m_sudo = new QRadioButton( i18n( "sudo" ), suid_progs, "SudoButton" );
-  
+  m_sudo = new QRadioButton( "sudo", suid_progs, "SudoButton" );
+
   QButtonGroup *suid_actions = new QButtonGroup( 1, QButtonGroup::Horizontal, i18n( "Actions" ), this, "SUIDActions" );
   m_unmount_dead = new QCheckBox( i18n( "Use super user privileges to force the unmounting of (broken) shares\n(Requires Linux kernel 2.4.11 or later)" ), suid_actions );
   m_use_suid = new QCheckBox( i18n( "Use super user privileges to mount and unmount shares" ), suid_actions );
-  
+
   QWidget *remove_widget = new QWidget( suid_actions );
   QGridLayout *grid1 = new QGridLayout( remove_widget );
   m_remove = new KPushButton( i18n( "Remove Entries" ), remove_widget );
   QSpacerItem *spacer1 = new QSpacerItem( 0, 0, QSizePolicy::Expanding, QSizePolicy::Preferred );
-  grid1->addItem( spacer1, 0, 0 );  
+  grid1->addItem( spacer1, 0, 0 );
   grid1->addWidget( m_remove, 0, 1, 0 );
-  
+
   QSpacerItem *spacer = new QSpacerItem( 0, 0, QSizePolicy::Preferred, QSizePolicy::Expanding );
-  
+
   grid->addWidget( suid_progs, 0, 0, 0 );
   grid->addWidget( suid_actions, 1, 0, 0 );
   grid->addItem( spacer, 2, 0 );
