@@ -7,24 +7,32 @@
  ***************************************************************************/
 
 /***************************************************************************
- *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
+ *   This program is distributed in the hope that it will be useful, but   *
+ *   WITHOUT ANY WARRANTY; without even the implied warranty of            *
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
+ *   General Public License for more details.                              *
+ *                                                                         *
+ *   You should have received a copy of the GNU General Public License     *
+ *   along with this program; if not, write to the                         *
+ *   Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
+ *   MA  02111-1307 USA                                                    *
  ***************************************************************************/
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
-#endif 
+#endif
 
 #include <iostream>
 #include <fstream>
 #include <cstring>
 #include <cerrno>
 #include <unistd.h>
-using namespace std; 
+using namespace std;
 
 
 //
@@ -52,12 +60,12 @@ const char *findprog( const char *name )
 {
   const char *paths[] = { "/bin/", "/sbin/", "/usr/bin/", "/usr/sbin/", "/usr/local/bin/", "/usr/local/sbin/" };
   string file = "";
-  
+
   for ( uint i = 0; i < sizeof( paths ) / sizeof( char * ); i++ )
   {
     string path( paths[i] );
     path.append( name );
-      
+
     if ( access( path.c_str(), X_OK ) == 0 )
     {
       file.assign( path );
@@ -70,7 +78,7 @@ const char *findprog( const char *name )
     cerr << "smb4k_kill: Could not find " << name << "." << endl;
     exit( EXIT_FAILURE );
   }
-    
+
   return file.c_str();
 }
 
@@ -85,15 +93,15 @@ int main( int argc, char *argv[], char *envp[] )
     info();
     exit( EXIT_FAILURE );
   }
-  
+
   char *args[25];
   int k = 0;
-  
-  char *p = (char *)malloc( 100*sizeof(char) );
-  
+
+  char *p = new char[100];
+
   args[k] = strcpy( p, findprog( "kill" ) );
   k++;
-  
+
   for ( int i = 1; i < argc; i++ )
   {
     if ( !strcmp( argv[i], "--help" ) )
@@ -102,37 +110,37 @@ int main( int argc, char *argv[], char *envp[] )
       exit( EXIT_SUCCESS );
     }
     else
-    {    
-      char *t = (char *)malloc( 100*sizeof(char) );
-      
+    {
+      char *t = new char[100];
+
       if ( !t )
       {
         cerr << "smb4k_kill: Out of memory" << endl;
         exit( EXIT_FAILURE );
       }
-      
+
       if ( strlen( argv[i] ) > 100 )
       {
         cerr << "smb4k_kill: One or more arguments exceed 100 characters." << endl;
         exit( EXIT_FAILURE );
       }
-                  
+
       args[k] = strcpy( t, argv[i] );
       k++;
-      
-      continue;    
+
+      continue;
     }
   }
-  
+
   args[k] = NULL;
-  
+
   if ( execve( args[0], args, envp ) == -1 )
   {
     int err_code = errno;
-    
+
     cerr << "smb4k_kill: " << strerror( err_code ) << endl;
     exit( EXIT_FAILURE );
   }
-  
+
   return EXIT_SUCCESS;
 }
