@@ -7,12 +7,20 @@
  ***************************************************************************/
 
 /***************************************************************************
- *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
+ *   This program is distributed in the hope that it will be useful, but   *
+ *   WITHOUT ANY WARRANTY; without even the implied warranty of            *
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
+ *   General Public License for more details.                              *
+ *                                                                         *
+ *   You should have received a copy of the GNU General Public License     *
+ *   along with this program; if not, write to the                         *
+ *   Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
+ *   MA  02111-1307 USA                                                    *
  ***************************************************************************/
 
 #ifdef HAVE_CONFIG_H
@@ -34,7 +42,7 @@ using namespace std;
 void info()
 {
   cout << "This is smb4k_mount, the mount utility for Smb4K." << endl;
-  cout << "(c) 2004-2005, Alexander Reinholdt" << endl << endl;
+  cout << "(c) 2004-2006, Alexander Reinholdt" << endl << endl;
   cout << "Usage:" << endl;
   cout << " smb4k_mount [--no-suid|--suid] <options>" << endl;
   cout << " smb4k_mount --help" << endl << endl;
@@ -43,7 +51,9 @@ void info()
   cout << "  --suid\tsmb4k_mount invokes mount." << endl;
   cout << "  --help\tDisplays this help screen and exits." << endl;
   cout << "  <options>\tThese are the options that you have to pass to the mount" << endl;
-  cout << "\t\tbinary." << endl;
+  cout << "\t\tbinary. Note, that under Linux you must specify the \'-t\'" << endl;
+  cout << "\t\toption under any circumstances. Under FreeBSD, however, it" << endl;
+  cout << "\t\tis not recognized and might cause problems." << endl;
   cout << endl;
 }
 
@@ -56,12 +66,12 @@ const char *findprog( const char *name )
 {
   const char *paths[] = { "/bin/", "/sbin/", "/usr/bin/", "/usr/sbin/", "/usr/local/bin/", "/usr/local/sbin/" };
   string file = "";
-  
+
   for ( uint i = 0; i < sizeof( paths ) / sizeof( char * ); i++ )
   {
     string path( paths[i] );
     path.append( name );
-      
+
     if ( access( path.c_str(), X_OK ) == 0 )
     {
       file.assign( path );
@@ -74,7 +84,7 @@ const char *findprog( const char *name )
     cerr << "smb4k_mount: Could not find " << name << "." << endl;
     exit( EXIT_FAILURE );
   }
-  
+
   return file.c_str();
 }
 
@@ -88,7 +98,7 @@ const char *findprog( const char *name )
 const char *replace_special_characters( const char *str )
 {
   string s( str );
- 
+
   for ( uint i = 0; i < s.length(); i++ )
   {
     switch ( s[i] )
@@ -232,7 +242,7 @@ const char *replace_special_characters( const char *str )
         break;
     }
   }
-  
+
   return s.c_str();
 }
 
@@ -248,12 +258,13 @@ int main( int argc, char *argv[], char *envp[] )
     info();
     exit( EXIT_FAILURE );
   }
-  
+
   bool fs_exists = false;
   bool suid_arg_exists = false;
+  bool use_suid = false;
   char *args[25];
   int k = 0;
-  
+
   // Loop through the argument
   for ( int i = 1; i < argc; i++ )
   {
@@ -264,48 +275,57 @@ int main( int argc, char *argv[], char *envp[] )
     }
     else if ( !strcmp( argv[i], "--no-suid" ) )
     {
-      char *p = (char *)malloc( 100*sizeof(char) );
-      
+      // Under FreeBSD, we can put the binary in the array already
+      // at this point, because there is only one. Under Linux, we
+      // have to determine the filesystem type before.
+
+#ifdef __FreeBSD__
+      char *p = new char[100];
+
       if ( !p )
       {
         cerr << "smb4k_mount: Out of memory" << endl;
         exit( EXIT_FAILURE );
       }
-      
-#ifndef __FreeBSD__
-      args[k] = strcpy( p, findprog( "smbmount" ) );
-#else
+
       args[k] = strcpy( p, findprog( "mount_smbfs" ) );
 #endif
+
       k++;
-      fs_exists = true;
       suid_arg_exists = true;
+      use_suid = false;
       continue;
     }
     else if ( !strcmp( argv[i], "--suid" ) )
     {
-      char *p = (char *)malloc( 100*sizeof(char) );
-      
+      // Under FreeBSD, we can put the binary in the array already
+      // at this point, because there is only one. Under Linux, we
+      // have to determine the filesystem type before.
+
+#ifdef __FreeBSD__
+      char *p = new char[100];
+
       if ( !p )
       {
         cerr << "smb4k_mount: Out of memory" << endl;
         exit( EXIT_FAILURE );
       }
-          
-#ifndef __FreeBSD__
-      args[k] = strcpy( p, findprog( "mount" ) );
-#else
+
       args[k] = strcpy( p, findprog( "mount_smbfs" ) );
 #endif
+
       k++;
       suid_arg_exists = true;
+      use_suid = true;
       continue;
     }
     else
     {
 #ifndef __FreeBSD__
       if ( !strcmp( argv[i], "-t" ) )
-      { 
+      {
+        fs_exists = true;
+
         if ( strcmp( argv[i+1], "smbfs" ) != 0 && strcmp( argv[i+1], "cifs" ) != 0 )
         {
           cerr << "smb4k_mount: You specified an invalid filesystem." << endl;
@@ -313,55 +333,97 @@ int main( int argc, char *argv[], char *envp[] )
         }
         else
         {
-          fs_exists = true;
+          // Put the binary into the first position.
+
+          if ( suid_arg_exists )
+          {
+            if ( use_suid )
+            {
+              char *p = new char[100];
+
+              if ( !p )
+              {
+                cerr << "smb4k_mount: Out of memory" << endl;
+                exit( EXIT_FAILURE );
+              }
+
+              args[0] = strcpy( p, findprog( "mount" ) );
+            }
+            else
+            {
+              char *p = new char[100];
+
+              if ( !p )
+              {
+                cerr << "smb4k_mount: Out of memory" << endl;
+                exit( EXIT_FAILURE );
+              }
+
+              if ( !strcmp( argv[i+1], "smbfs" ) )
+              {
+                args[0] = strcpy( p, findprog( "smbmount" ) );
+              }
+              else if ( !strcmp( argv[i+1], "cifs" ) )
+              {
+                args[0] = strcpy( p, findprog( "mount.cifs" ) );
+              }
+
+              // For smbmount and mount.cifs, we do not need the -t option.
+              // Jump to the next argument.
+              i++;
+
+              continue;
+            }
+          }
+          else
+          {
+            cerr << "smb4k_mount: You have to supply either the '--suid' or '--no-suid' argument." << endl;
+            exit( EXIT_FAILURE );
+          }
         }
       }
 #endif
-      
-      char *t = (char *)malloc( 250*sizeof(char) );
-      
+
+      char *t = new char[strlen( argv[i] )+1];
+
       if ( !t )
       {
         cerr << "smb4k_mount: Out of memory" << endl;
         exit( EXIT_FAILURE );
       }
-      
-      if ( strlen( argv[i] ) > 250 )
-      {
-        cerr << "smb4k_mount: One or more arguments exceed the limit of 250 characters!" << endl;
-        exit( EXIT_FAILURE );
-      }
-            
+
       args[k] = strcpy( t, replace_special_characters( argv[i] ) );
       k++;
-      
+
       continue;
     }
   }
-  
+
   args[k] = NULL;
-  
+
+  // FIXME: Do we need the following under FreeBSD?
+
   if ( !suid_arg_exists )
   {
     cerr << "smb4k_mount: You have to supply either the '--suid' or '--no-suid' argument." << endl;
     exit( EXIT_FAILURE );
   }
 
-#ifndef __FreeBSD__    
+#ifndef __FreeBSD__
   if ( !fs_exists )
   {
     cerr << "smb4k_mount: You haven't specified a filesystem (smbfs or cifs)." << endl;
     exit( EXIT_FAILURE );
   }
 #endif
-  
+
   if ( execve( args[0], args, envp ) == -1 )
   {
     int err_code = errno;
-    
+
     cerr << "smb4k_mount: " << strerror( err_code ) << endl;
     exit( EXIT_FAILURE );
   }
-  
+
   return EXIT_SUCCESS;
 }
