@@ -7,12 +7,20 @@
  ***************************************************************************/
 
 /***************************************************************************
- *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
+ *   This program is distributed in the hope that it will be useful, but   *
+ *   WITHOUT ANY WARRANTY; without even the implied warranty of            *
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
+ *   General Public License for more details.                              *
+ *                                                                         *
+ *   You should have received a copy of the GNU General Public License     *
+ *   along with this program; if not, write to the                         *
+ *   Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
+ *   MA  02111-1307 USA                                                    *
  ***************************************************************************/
 
 #ifdef HAVE_CONFIG_H
@@ -24,20 +32,19 @@
 #include <cstring>
 #include <cerrno>
 #include <unistd.h>
-#ifndef __FreeBSD__
+
+#if !defined(__FreeBSD__) && !defined(__solaris__) && !defined(USE_SOLARIS)
 #include <sys/statfs.h>
-#else
+#elif defined(__solaris__) || defined(USE_SOLARIS)
+#include <sys/types.h>
+#include <sys/statvfs.h>
+#elif defined(__FreeBSD__)
 #include <sys/param.h>
 #include <sys/mount.h>
 #endif
+
 using namespace std;
 
-//
-// Global variable that carries the name of the file system
-//
-
-char fs[5];
-
 //
 // Returns the information about the program
 //
@@ -45,15 +52,15 @@ char fs[5];
 void info()
 {
   cout << "This is smb4k_umount, the unmount utility for Smb4K." << endl;
-  cout << "(c) 2004-2005, Alexander Reinholdt" << endl << endl;
+  cout << "(c) 2004-2006, Alexander Reinholdt" << endl << endl;
   cout << "Usage:" << endl;
   cout << " smb4k_umount [--no-suid|--suid] [--cifs|--smbfs] <mountpoint>" << endl;
   cout << " smb4k_umount --help" << endl << endl;
   cout << "Arguments:" << endl;
-  cout << "  --no-suid\tsmb4k_umount invokes smbumount." << endl;
+  cout << "  --no-suid\tsmb4k_umount invokes smbumount or umount.cifs." << endl;
   cout << "  --suid\tsmb4k_umount invokes umount." << endl;
-  cout << "  --cifs\tUnmount a CIFS share." << endl;
-  cout << "  --smbfs\tUnmount an SMBFS share." << endl;
+  cout << "  --cifs\tThe share has been mounted with the CIFS file system." << endl;
+  cout << "  --smbfs\tThe share has been mounted with the SMBFS file system." << endl;
   cout << "  --help\tDisplays this help screen and exits." << endl;
   cout << "  <mountpoint>\tThe mountpoint of the share." << endl;
   cout << endl;
@@ -253,11 +260,19 @@ const char *replace_special_characters( const char *str )
 // Checks wether the user is allowed to unmount the
 // specified filesystem
 //
-bool check( const char *path )
+bool check( const char *path, char *fs_buf )
 {
+#if !defined(__solaris__) && !defined(USE_SOLARIS)
   struct statfs filesystem;
+#else
+  struct statvfs filesystem;
+#endif
 
+#if !defined(__solaris__) && !defined(USE_SOLARIS)
   if ( statfs( path, &filesystem ) == -1 )
+#else
+  if ( statvfs( path, &filesystem ) == -1 )
+#endif
   {
     int err_code = errno;
 
@@ -268,25 +283,28 @@ bool check( const char *path )
     }
     else
     {
-      return true; // Bypass the check below, because it would yield ok = FALSE
+      fs_buf[0] = '\0';
+
+      return true; // Bypass the check below, because it would yield ok == FALSE
                    // and we want to be able to unmount broken shares as well.
     }
   }
 
   bool ok = false;
 
-#ifndef __FreeBSD__
+#if !defined(__FreeBSD__) && !defined(__solaris__) && !defined(USE_SOLARIS)
   // First entry is for CIFS, the second for SMBFS.
   if ( (uint)filesystem.f_type == 0xFF534D42 || (uint)filesystem.f_type == 0x517B )
   {
-//     if ( ((uint)filesystem.f_type == 0xFF534D42 && strcmp( fs, "cifs" )) ||
-//          ((uint)filesystem.f_type == 0x517B && strcmp( fs, "smbfs" )) )
-//     {
-//       cerr << "smb4k_umount: Wrong filesystem argument! Unmounting might fail." << endl;
-//     }
-#else
+    strcpy( fs_buf, (uint)filesystem.f_type == 0xFF534D42 ? "cifs" : "smbfs" );
+#elif defined(__FreeBSD__)
   if ( !strcmp( filesystem.f_fstypename, "smbfs" ) )
   {
+    strcpy( fs_buf, "smbfs" );
+#elif defined(__solaris__) || defined(USE_SOLARIS)
+  if ( (uint)filesystem.f_basetype == 0xFF534D42 || (uint)filesystem.f_basetype == 0x517B )
+  {
+    strcpy( fs_buf, (uint)filesystem.f_basetype == 0xFF534D42 ? "cifs" : "smbfs" );
 #endif
     ok = true;
   }
@@ -308,20 +326,11 @@ int main( int argc, char *argv[], char *envp[] )
     exit( EXIT_FAILURE );
   }
 
-  char *args[25];
   int k = 0;
-  char *path = (char *)malloc( 250*sizeof(char) );
-
-  if ( !path )
-  {
-    cerr << "smb4k_umount: Out of memory" << endl;
-    exit( EXIT_FAILURE );
-  }
-
   bool suid_arg_exists = false;
-  bool fs_arg_exists = false;
-
-  int pos;
+  bool use_suid = false;
+  char *args[5];
+  char fs_arg[10];
 
   for ( int i = 1; i < argc; i++ )
   {
@@ -332,115 +341,148 @@ int main( int argc, char *argv[], char *envp[] )
     }
     else if ( !strcmp( argv[i], "--no-suid" ) )
     {
-      char *p = (char *)malloc( 100*sizeof(char) );
-
-      if ( !p )
-      {
-        cerr << "smb4k_umount: Out of memory" << endl;
-        exit( EXIT_FAILURE );
-      }
-
-      args[k] = strcpy( p, findprog( "smbumount" ) );
-      pos = k;
-      k++;
       suid_arg_exists = true;
+      use_suid = false;
+      // Leave first entry in args[] free:
+      k++;
+
       continue;
     }
     else if ( !strcmp( argv[i], "--suid" ) )
     {
-      char *p = (char *)malloc( 100*sizeof(char) );
-
-      if ( !p )
-      {
-        cerr << "smb4k_umount: Out of memory" << endl;
-        exit( EXIT_FAILURE );
-      }
-
-      args[k] = strcpy( p, findprog( "umount" ) );
-      pos = k;
-      k++;
       suid_arg_exists = true;
+      use_suid = true;
+      // Leave first entry in args[] free:
+      k++;
+
       continue;
     }
-    else if ( !strcmp( argv[i], "--smbfs" ) )
+    else if ( !strcmp( argv[i], "--cifs" ) || !strcmp( argv[i], "--smbfs" ) )
     {
-      fs_arg_exists = true;
-      strcpy( fs, "smbfs" );
-      continue;
-    }
-    else if ( !strcmp( argv[i], "--cifs" ) )
-    {
-      fs_arg_exists = true;
-      strcpy( fs, "cifs" );
-
-      if ( strcmp( args[pos], findprog( "umount" ) ) )
-      {
-        free( args[pos] );
-
-        char *p = (char *)malloc( 100*sizeof(char) );
-
-        if ( !p )
-        {
-          cerr << "smb4k_umount: Out of memory" << endl;
-          exit( EXIT_FAILURE );
-        }
-
-        args[k] = strcpy( p, findprog( "umount" ) );
-      }
+      (void) strcpy( fs_arg, argv[i] );
 
       continue;
     }
     else
     {
-      char *t = (char *)malloc( 250*sizeof(char) );
+      // Only do something, if the argument is a mount point:
 
-      if ( !t )
+      // FIXME: Honor relative paths etc. as well.
+
+      if ( argv[i][0] == '/' || ((argv[i][0] == '\047' /* ' */|| argv[i][0] == '\042' /* " */) && argv[i][1] == '/') )
       {
-        cerr << "smb4k_umount: Out of memory" << endl;
-        exit( EXIT_FAILURE );
-      }
+        char *path = new char[strlen( argv[i] )+1];
 
-      if ( strlen( argv[i] ) > 250 )
+        if ( !path )
+        {
+          cerr << "smb4k_umount: Out of memory" << endl;
+          exit( EXIT_FAILURE );
+        }
+
+        args[k++] = strcpy( path, replace_special_characters( argv[i] ) );
+
+        char fs[6];
+
+        if ( check( path, fs ) )
+        {
+          char *t = new char[255];
+
+          if ( !t )
+          {
+            cerr << "smb4k_umount: Out of memory" << endl;
+            exit( EXIT_FAILURE );
+          }
+
+          if ( suid_arg_exists )
+          {
+            if ( !use_suid )
+            {
+              if ( !strcmp( fs, "smbfs" ) )
+              {
+                args[0] = strcpy( t, findprog( "smbumount" ) );
+              }
+              else if ( !strcmp( fs, "cifs" ) )
+              {
+                args[0] = strcpy( t, findprog( "umount.cifs" ) );
+              }
+              else
+              {
+                // If the check failed, we need to use the user supplied
+                // file system info.
+                if ( !strcmp( fs_arg, "--smbfs" ) )
+                {
+                  args[0] = strcpy( t, findprog( "smbumount" ) );
+                }
+                else if ( !strcmp( fs_arg, "--cifs" ) )
+                {
+                  args[0] = strcpy( t, findprog( "umount.cifs" ) );
+                }
+                else
+                {
+                  delete [] t;
+
+                  cerr << "smb4k_umount: Could not determine umount binary" << endl;
+
+                  exit( EXIT_FAILURE );
+                }
+              }
+            }
+            else
+            {
+              if ( !strcmp( fs, "smbfs" ) || !strcmp( fs, "cifs" ) )
+              {
+                args[0] = strcpy( t, findprog( "umount" ) );
+              }
+              else
+              {
+                if ( !strcmp( fs_arg, "--smbfs" ) || !strcmp( fs_arg, "--cifs" ) )
+                {
+                  args[0] = strcpy( t, findprog( "umount" ) );
+                }
+                else
+                {
+                  delete [] t;
+
+                  cerr << "smb4k_umount: Could not determine umount binary" << endl;
+
+                  exit( EXIT_FAILURE );
+                }
+              }
+            }
+          }
+          else
+          {
+            cerr << "smb4k_mount: You have neither supplied the '--suid' nor '--no-suid' argument." << endl;
+            exit( EXIT_FAILURE );
+          }
+        }
+        else
+        {
+          cerr << "smb4k_umount: You are not allowed to unmount this filesystem type." << endl;
+          exit( EXIT_FAILURE );
+        }
+
+        continue;
+      }
+      else
       {
-        cerr << "smb4k_umount: One or more arguments exceed 250 characters." << endl;
-        exit( EXIT_FAILURE );
+        char *t = new char[strlen( argv[i] )+1];
+
+        if ( !t )
+        {
+          cerr << "smb4k_umount: Out of memory" << endl;
+          exit( EXIT_FAILURE );
+        }
+
+        args[k++] = strcpy( t, argv[i] );
+
+        continue;
       }
-
-      args[k] = strcpy( t, replace_special_characters( argv[i] ) );
-
-      if ( args[k][0] == '/' )
-      {
-        path = strcpy( path, args[k] );
-      }
-
-      k++;
-
-      continue;
     }
   }
 
   args[k] = NULL;
 
-  if ( !suid_arg_exists )
-  {
-    cerr << "smb4k_mount: You have neither supplied the '--suid' nor '--no-suid' argument." << endl;
-    exit( EXIT_FAILURE );
-  }
-
-  if ( !fs_arg_exists )
-  {
-    cerr << "smb4k_umount: The filesystem argument is missing." << endl;
-    exit( EXIT_FAILURE );
-  }
-
-  if ( !check( path ) )
-  {
-    cerr << "smb4k_umount: You are not allowed to unmount this filesystem type." << endl;
-    exit( EXIT_FAILURE );
-  }
-
-  free( path );
-
   if ( execve( args[0], args, envp ) == -1 )
   {
     int err_code = errno;
